@@ -1,7 +1,7 @@
 //! Genesis public interface and main loop
 
-use crate::audio::{GenesisAudioResampler, LowPassFilter};
-use crate::input::{GenesisInputs, InputState};
+use crate::audio::GenesisAudioResampler;
+use crate::input::{GenesisButton, GenesisInputs, InputState};
 use crate::memory::{Cartridge, MainBus, MainBusSignals, MainBusWrites, Memory};
 use crate::timing::{CycleCounters, GenesisCycleCounters};
 use crate::vdp::{TimingModeExt, Vdp, VdpConfig, VdpTickEffect};
@@ -9,11 +9,11 @@ use crate::ym2612::{Ym2612, YmTickEffect};
 use crate::{GenesisControllerType, audio, timing, vdp};
 use bincode::{Decode, Encode};
 use jgenesis_common::frontend::{
-    AudioOutput, Color, EmulatorTrait, FrameSize, PartialClone, PixelAspectRatio, Renderer,
-    SaveWriter, TickEffect, TimingMode,
+    AudioOutput, Color, EmulatorConfigTrait, EmulatorTrait, FrameSize, PartialClone,
+    PixelAspectRatio, Renderer, SaveWriter, TickEffect, TimingMode,
 };
 use jgenesis_common::num::GetBit;
-use jgenesis_proc_macros::{EnumAll, EnumDisplay, EnumFromStr};
+use jgenesis_proc_macros::{ConfigDisplay, EnumAll, EnumDisplay, EnumFromStr};
 use m68000_emu::M68000;
 use smsgg_core::psg::{Sn76489, Sn76489TickEffect, Sn76489Version};
 use std::cmp;
@@ -135,7 +135,16 @@ impl GenesisRegion {
     }
 }
 
-#[derive(Debug, Clone, Copy, Encode, Decode)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Encode, Decode, EnumAll, EnumDisplay)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "clap", derive(jgenesis_proc_macros::CustomValueEnum))]
+pub enum GenesisLowPassFilter {
+    None,
+    #[default]
+    Model1Va2,
+}
+
+#[derive(Debug, Clone, Copy, Encode, Decode, ConfigDisplay)]
 pub struct GenesisEmulatorConfig {
     pub p1_controller_type: GenesisControllerType,
     pub p2_controller_type: GenesisControllerType,
@@ -146,6 +155,7 @@ pub struct GenesisEmulatorConfig {
     pub remove_sprite_limits: bool,
     pub m68k_clock_divider: u64,
     pub emulate_non_linear_vdp_dac: bool,
+    pub deinterlace: bool,
     pub render_vertical_border: bool,
     pub render_horizontal_border: bool,
     pub plane_a_enabled: bool,
@@ -155,7 +165,7 @@ pub struct GenesisEmulatorConfig {
     pub backdrop_enabled: bool,
     pub quantize_ym2612_output: bool,
     pub emulate_ym2612_ladder_effect: bool,
-    pub low_pass_filter: LowPassFilter,
+    pub low_pass: GenesisLowPassFilter,
     pub ym2612_enabled: bool,
     pub psg_enabled: bool,
 }
@@ -166,6 +176,7 @@ impl GenesisEmulatorConfig {
         VdpConfig {
             enforce_sprite_limits: !self.remove_sprite_limits,
             emulate_non_linear_dac: self.emulate_non_linear_vdp_dac,
+            deinterlace: self.deinterlace,
             render_vertical_border: self.render_vertical_border,
             render_horizontal_border: self.render_horizontal_border,
             plane_a_enabled: self.plane_a_enabled,
@@ -190,6 +201,12 @@ impl GenesisEmulatorConfig {
     }
 }
 
+impl EmulatorConfigTrait for GenesisEmulatorConfig {
+    fn with_overclocking_disabled(&self) -> Self {
+        Self { m68k_clock_divider: timing::NATIVE_M68K_DIVIDER, ..*self }
+    }
+}
+
 #[derive(Debug, Encode, Decode, PartialClone)]
 pub struct GenesisEmulator {
     #[partial_clone(partial)]
@@ -333,6 +350,7 @@ pub fn render_frame<R: Renderer>(
 }
 
 impl EmulatorTrait for GenesisEmulator {
+    type Button = GenesisButton;
     type Inputs = GenesisInputs;
     type Config = GenesisEmulatorConfig;
 
@@ -393,8 +411,9 @@ impl EmulatorTrait for GenesisEmulator {
 
         while self.cycles.should_tick_psg() {
             if self.psg.tick() == Sn76489TickEffect::Clocked {
-                let (psg_sample_l, psg_sample_r) = self.psg.sample();
-                self.audio_resampler.collect_psg_sample(psg_sample_l, psg_sample_r);
+                // PSG only has mono output in the Genesis; stereo output is only for Game Gear
+                let (psg_sample, _) = self.psg.sample();
+                self.audio_resampler.collect_psg_sample(psg_sample);
             }
 
             self.cycles.decrement_psg();
@@ -472,10 +491,6 @@ impl EmulatorTrait for GenesisEmulator {
         *self = GenesisEmulator::create(rom, self.config, save_writer);
     }
 
-    fn timing_mode(&self) -> TimingMode {
-        self.timing_mode
-    }
-
     fn target_fps(&self) -> f64 {
         target_framerate(self.timing_mode)
     }
