@@ -122,6 +122,8 @@ pub struct Cartridge {
 const TRIPLE_PLAY_GOLD_SERIAL: &[u8] = b"T-172116";
 const TRIPLE_PLAY_96_SERIAL: &[u8] = b"T-172026";
 
+const QUACKSHOT_REV_A_SERIAL: &[u8] = b"GM 00004054-01";
+
 const ROCKMAN_X3_CHECKSUM: u32 = 0x3EE639F0;
 
 impl Cartridge {
@@ -134,7 +136,7 @@ impl Cartridge {
         let checksum = CRC.checksum(&rom_bytes);
         log::info!("ROM CRC32: {checksum:08X}");
 
-        let mut rom_bytes = ensure_big_endian(rom_bytes);
+        let mut rom_bytes = ensure_rom_in_expected_format(rom_bytes);
 
         let region = forced_region.unwrap_or_else(|| {
             GenesisRegion::from_rom(&rom_bytes).unwrap_or_else(|| {
@@ -161,14 +163,11 @@ impl Cartridge {
         if rom_bytes.len() >= 0x300000
             && (serial_number == TRIPLE_PLAY_GOLD_SERIAL || serial_number == TRIPLE_PLAY_96_SERIAL)
         {
-            // Triple Play expects the third MB of the ROM to be mapped to $300000-$3FFFFF instead
-            // of $200000-$2FFFFF; accomplish this by duplicating the data
-            if rom_bytes.len() < 0x400000 {
-                rom_bytes.extend(iter::repeat(0xFF).take(0x400000 - rom_bytes.len()));
-            }
+            fix_triple_play_rom(&mut rom_bytes);
+        }
 
-            let (first, second) = rom_bytes.split_at_mut(0x300000);
-            second[..0x100000].copy_from_slice(&first[0x200000..0x300000]);
+        if rom_bytes.len() == 0x80000 && &rom_bytes[0x180..0x18E] == QUACKSHOT_REV_A_SERIAL {
+            rom_bytes = fix_quackshot_rev_a_rom(rom_bytes);
         }
 
         let is_unlicensed_rockman_x3 = checksum == ROCKMAN_X3_CHECKSUM;
@@ -232,6 +231,39 @@ impl Cartridge {
     }
 }
 
+fn ensure_rom_in_expected_format(rom: Vec<u8>) -> Vec<u8> {
+    let rom = remove_copier_header(rom);
+    let rom = deinterleave_rom(rom);
+    ensure_big_endian(rom)
+}
+
+fn remove_copier_header(rom: Vec<u8>) -> Vec<u8> {
+    // Some older ROMs contain a useless 512-byte copier header; remove it if present
+    if rom.len() & 0x3FF != 0x200 {
+        // ROM length is not off by 512 from a reasonable number
+        return rom;
+    }
+
+    // TMSS header is normally at $100-$103, would be at $303-$304 with the 512-byte header
+    let tmss_header = &rom[0x300..0x304];
+
+    // Interleaved header bytes are normally at $80-$81 for even and $2080-$2081 for odd
+    let interleaved_tmss_even = &rom[0x0280..0x0282];
+    let interleaved_tmss_odd = &rom[0x2280..0x2282];
+
+    if tmss_header != b"SEGA"
+        && tmss_header != b"ESAG"
+        && !(interleaved_tmss_even == b"EA" && interleaved_tmss_odd == b"SG")
+    {
+        // Removing the copier header would not produce a valid TMSS header
+        return rom;
+    }
+
+    log::info!("ROM image appears to have a 512-byte copier header; removing it");
+
+    rom.into_iter().skip(512).collect()
+}
+
 fn ensure_big_endian(mut rom: Vec<u8>) -> Vec<u8> {
     // Every licensed game contains the ASCII string "SEGA" at $100-$104 in ROM
     // If the string "ESAG" is detected there, byteswap the ROM
@@ -246,6 +278,63 @@ fn ensure_big_endian(mut rom: Vec<u8>) -> Vec<u8> {
     rom
 }
 
+fn deinterleave_rom(rom: Vec<u8>) -> Vec<u8> {
+    // Some older ROM images, usually with the .smd file extension, are interleaved.
+    // This format consists of 16KB blocks where each block contains 8KB of even bytes followed by
+    // 8KB of odd bytes.
+    if rom.len() % (16 * 1024) != 0 {
+        // Interleaved ROM sizes should always be a multiple of 16KB
+        return rom;
+    }
+
+    if &rom[0x100..0x104] == b"SEGA" || &rom[0x100..0x104] == b"ESAG" {
+        // ROM image already contains valid TMSS text; don't try to deinterleave
+        return rom;
+    }
+
+    if &rom[0x0080..0x0082] != b"EA" || &rom[0x2080..0x2082] != b"SG" {
+        // Deinterleaving would not produce valid TMSS text; don't try to deinterleave
+        return rom;
+    }
+
+    log::info!("ROM image appears to be interleaved; deinterleaving it");
+
+    let mut deinterleaved = vec![0; rom.len()];
+    for block_addr in (0..rom.len()).step_by(0x4000) {
+        for i in 0..0x2000 {
+            deinterleaved[block_addr + 2 * i] = rom[block_addr + 0x2000 + i];
+            deinterleaved[block_addr + 2 * i + 1] = rom[block_addr + i];
+        }
+    }
+
+    deinterleaved
+}
+
+fn fix_triple_play_rom(rom: &mut Vec<u8>) {
+    // Triple Play expects the third MB of the ROM to be mapped to $300000-$3FFFFF instead
+    // of $200000-$2FFFFF; accomplish this by duplicating the data
+    if rom.len() < 0x400000 {
+        rom.extend(iter::repeat(0xFF).take(0x400000 - rom.len()));
+    }
+
+    let (first, second) = rom.split_at_mut(0x300000);
+    second[..0x100000].copy_from_slice(&first[0x200000..0x300000]);
+}
+
+fn fix_quackshot_rev_a_rom(rom: Vec<u8>) -> Vec<u8> {
+    // QuackShot (Rev A) is a 512KB ROM with an unusual ROM address mapping:
+    //   $000000-$0FFFFF: First 256KB of ROM mirrored 4x
+    //   $100000-$1FFFFF: Second 256KB of ROM mirrored 4x
+    // Rather than implement custom mapping logic, just remap the ROM while loading it
+    let mut remapped_rom = vec![0; 0x200000];
+    for i in (0x000000..0x100000).step_by(0x40000) {
+        remapped_rom[i..i + 0x40000].copy_from_slice(&rom[..0x40000]);
+        remapped_rom[i + 0x100000..i + 0x140000].copy_from_slice(&rom[0x40000..]);
+    }
+
+    remapped_rom
+}
+
 #[must_use]
 #[allow(clippy::missing_panics_doc, clippy::items_after_statements)]
 pub fn parse_title_from_header(rom: &[u8], region: GenesisRegion) -> String {
@@ -696,13 +785,16 @@ impl<'a, Medium: PhysicalMedium> MainBus<'a, Medium> {
                 self.memory.physical_medium.write_byte(address, value);
             }
             0xA00000..=0xA0FFFF => {
-                // Z80 memory map
-                // For 68k access, $8000-$FFFF mirrors $0000-$7FFF
-                <Self as z80_emu::BusInterface>::write_memory(
-                    self,
-                    (address & 0x7FFF) as u16,
-                    value,
-                );
+                // Z80 memory map; writable by the 68k only when the Z80 is removed from the bus
+                // and not reset
+                if self.memory.signals.z80_busack() {
+                    // For 68k access, $8000-$FFFF mirrors $0000-$7FFF
+                    <Self as z80_emu::BusInterface>::write_memory(
+                        self,
+                        (address & 0x7FFF) as u16,
+                        value,
+                    );
+                }
             }
             0xA10000..=0xA1001F => {
                 self.write_io_register(address, value);
@@ -796,9 +888,14 @@ impl<Medium: PhysicalMedium> m68000_emu::BusInterface for MainBus<'_, Medium> {
                 self.memory.physical_medium.read_byte(address)
             }
             0xA00000..=0xA0FFFF => {
-                // Z80 memory map
-                // For 68k access, $8000-$FFFF mirrors $0000-$7FFF
-                <Self as z80_emu::BusInterface>::read_memory(self, (address & 0x7FFF) as u16)
+                // Z80 memory map; 68k can only access when the Z80 is running and removed from the bus
+                if self.memory.signals.z80_busack() {
+                    // For 68k access, $8000-$FFFF mirrors $0000-$7FFF
+                    <Self as z80_emu::BusInterface>::read_memory(self, (address & 0x7FFF) as u16)
+                } else {
+                    // MSB of open bus
+                    self.last_word_read.msb()
+                }
             }
             0xA10000..=0xA1001F => self.read_io_register(address),
             0xA11100..=0xA11101 => (self.read_busack_register() >> 8) as u8,
@@ -818,9 +915,15 @@ impl<Medium: PhysicalMedium> m68000_emu::BusInterface for MainBus<'_, Medium> {
                 self.memory.physical_medium.read_word(address)
             }
             0xA00000..=0xA0FFFF => {
-                // All Z80 access is byte-size; word reads mirror the byte in both MSB and LSB
-                let byte = self.read_byte(address);
-                u16::from_le_bytes([byte, byte])
+                // Z80 memory map; 68k can only access when the Z80 is running and removed from the bus
+                if self.memory.signals.z80_busack() {
+                    // All Z80 access is byte-size; word reads mirror the byte in both MSB and LSB
+                    let byte = self.read_byte(address);
+                    u16::from_le_bytes([byte, byte])
+                } else {
+                    // MSB is open bus MSB, LSB is 0
+                    self.last_word_read & 0xFF00
+                }
             }
             0xA10000..=0xA1001F => self.read_io_register(address).into(),
             0xA11100..=0xA11101 => self.read_busack_register(),
