@@ -4,15 +4,15 @@ use crate::bus::cartridge::CartridgeFileError;
 use crate::bus::{Bus, cartridge};
 use crate::cpu::CpuState;
 use crate::graphics::TimingModeGraphicsExt;
-use crate::input::NesInputs;
+use crate::input::{NesButton, NesInputs};
 use crate::ppu::PpuState;
 use crate::{apu, audio, cpu, graphics, ppu};
 use bincode::{Decode, Encode};
 use jgenesis_common::frontend::{
-    AudioOutput, Color, EmulatorTrait, FrameSize, PixelAspectRatio, Renderer, SaveWriter,
-    TickEffect, TickResult, TimingMode,
+    AudioOutput, Color, EmulatorConfigTrait, EmulatorTrait, FrameSize, PixelAspectRatio, Renderer,
+    SaveWriter, TickEffect, TickResult, TimingMode,
 };
-use jgenesis_proc_macros::{EnumAll, EnumDisplay, PartialClone};
+use jgenesis_proc_macros::{ConfigDisplay, EnumAll, EnumDisplay, PartialClone};
 use std::fmt::{Debug, Display, Formatter};
 use std::mem;
 use thiserror::Error;
@@ -71,7 +71,16 @@ impl Display for Overscan {
     }
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Encode, Decode)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Encode, Decode, EnumDisplay, EnumAll)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "clap", derive(jgenesis_proc_macros::CustomValueEnum))]
+pub enum NesAudioResampler {
+    LowPassNearestNeighbor,
+    #[default]
+    WindowedSinc,
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Encode, Decode, ConfigDisplay)]
 pub struct NesEmulatorConfig {
     /// Force timing mode to NTSC/PAL if set
     /// If None, timing mode will default based on iNES ROM header
@@ -87,6 +96,7 @@ pub struct NesEmulatorConfig {
     pub pal_black_border: bool,
     /// If true, silence the triangle wave channel when it is outputting a wave at ultrasonic frequency
     pub silence_ultrasonic_triangle_output: bool,
+    pub audio_resampler: NesAudioResampler,
     /// If true, adjust audio frequency so that audio sync times to 60Hz NTSC / 50Hz PAL
     pub audio_refresh_rate_adjustment: bool,
     /// Whether to allow simultaneous left+right and up+down joypad inputs.
@@ -95,6 +105,8 @@ pub struct NesEmulatorConfig {
     pub allow_opposing_joypad_inputs: bool,
 }
 
+impl EmulatorConfigTrait for NesEmulatorConfig {}
+
 #[derive(Debug, Error)]
 pub enum NesError<RErr, AErr, SErr> {
     #[error("Error rendering frame: {0}")]
@@ -157,7 +169,7 @@ impl NesEmulator {
             apu_state,
             config,
             rgba_frame_buffer: new_rgba_frame_buffer(),
-            audio_resampler: AudioResampler::new(timing_mode, config.audio_refresh_rate_adjustment),
+            audio_resampler: AudioResampler::new(timing_mode, &config),
             raw_rom_bytes: rom_bytes,
         })
     }
@@ -277,6 +289,7 @@ fn new_rgba_frame_buffer() -> Vec<Color> {
 }
 
 impl EmulatorTrait for NesEmulator {
+    type Button = NesButton;
     type Inputs = NesInputs;
     type Config = NesEmulatorConfig;
     type Err<
@@ -349,8 +362,7 @@ impl EmulatorTrait for NesEmulator {
         self.config = *config;
 
         self.bus.reload_config(*config);
-        self.audio_resampler
-            .set_apply_refresh_rate_adjustment(config.audio_refresh_rate_adjustment);
+        self.audio_resampler.reload_config(config);
     }
 
     fn take_rom_from(&mut self, other: &mut Self) {
@@ -376,12 +388,8 @@ impl EmulatorTrait for NesEmulator {
             .expect("Creation during hard reset should never fail");
     }
 
-    fn timing_mode(&self) -> TimingMode {
-        self.bus.mapper().timing_mode()
-    }
-
     fn target_fps(&self) -> f64 {
-        let timing_mode = self.timing_mode();
+        let timing_mode = self.bus.mapper().timing_mode();
         match (timing_mode, self.config.audio_refresh_rate_adjustment) {
             (TimingMode::Ntsc, true) => 60.0,
             (TimingMode::Ntsc, false) => audio::NTSC_NES_NATIVE_DISPLAY_RATE,
