@@ -5,16 +5,17 @@
 use crate::audio::Sega32XResampler;
 use crate::core::Sega32X;
 use bincode::{Decode, Encode};
-use genesis_core::input::InputState;
+use genesis_core::input::{GenesisButton, InputState};
 use genesis_core::memory::{MainBus, MainBusSignals, MainBusWrites, Memory};
 use genesis_core::timing::GenesisCycleCounters;
 use genesis_core::vdp::{Vdp, VdpTickEffect};
 use genesis_core::ym2612::{Ym2612, YmTickEffect};
 use genesis_core::{GenesisEmulatorConfig, GenesisInputs, GenesisRegion};
 use jgenesis_common::frontend::{
-    AudioOutput, Color, EmulatorTrait, Renderer, SaveWriter, TickEffect, TickResult, TimingMode,
+    AudioOutput, Color, EmulatorConfigTrait, EmulatorTrait, Renderer, SaveWriter, TickEffect,
+    TickResult, TimingMode,
 };
-use jgenesis_proc_macros::{EnumAll, EnumDisplay, PartialClone};
+use jgenesis_proc_macros::{ConfigDisplay, EnumAll, EnumDisplay, PartialClone};
 use m68000_emu::M68000;
 use smsgg_core::psg::{Sn76489, Sn76489TickEffect, Sn76489Version};
 use std::fmt::{Debug, Display};
@@ -42,13 +43,21 @@ pub enum S32XVideoOut {
     S32XOnly,
 }
 
-#[derive(Debug, Clone, Copy, Encode, Decode)]
+#[derive(Debug, Clone, Copy, Encode, Decode, ConfigDisplay)]
 pub struct Sega32XEmulatorConfig {
+    #[cfg_display(skip)]
     pub genesis: GenesisEmulatorConfig,
     pub video_out: S32XVideoOut,
+    pub apply_genesis_lpf_to_pwm: bool,
     pub pwm_enabled: bool,
 }
 
+impl EmulatorConfigTrait for Sega32XEmulatorConfig {
+    fn with_overclocking_disabled(&self) -> Self {
+        Self { genesis: self.genesis.with_overclocking_disabled(), ..*self }
+    }
+}
+
 macro_rules! new_main_bus {
     ($self:expr, m68k_reset: $m68k_reset:expr) => {
         MainBus::new(
@@ -146,7 +155,7 @@ impl Sega32XEmulator {
     #[must_use]
     pub fn cartridge_title(&self) -> String {
         genesis_core::memory::parse_title_from_header(
-            &self.memory.medium().cartridge.rom,
+            &self.memory.medium().cartridge().rom,
             self.region,
         )
     }
@@ -166,7 +175,7 @@ impl Sega32XEmulator {
     fn render_frame<R: Renderer>(&mut self, renderer: &mut R) -> Result<(), R::Err> {
         let frame_size = self.vdp.frame_size();
         let aspect_ratio = self.config.genesis.aspect_ratio.to_pixel_aspect_ratio(frame_size, true);
-        self.memory.medium().vdp.render_frame(
+        self.memory.medium_mut().vdp().render_frame(
             self.vdp.frame_buffer(),
             frame_size,
             aspect_ratio,
@@ -176,6 +185,7 @@ impl Sega32XEmulator {
 }
 
 impl EmulatorTrait for Sega32XEmulator {
+    type Button = GenesisButton;
     type Inputs = GenesisInputs;
     type Config = Sega32XEmulatorConfig;
     type Err<
@@ -235,8 +245,9 @@ impl EmulatorTrait for Sega32XEmulator {
 
         while self.cycles.should_tick_psg() {
             if self.psg.tick() == Sn76489TickEffect::Clocked {
-                let (sample_l, sample_r) = self.psg.sample();
-                self.audio_resampler.collect_psg_sample(sample_l, sample_r);
+                // PSG output is mono in Genesis; stereo output is only for Game Gear
+                let (psg_sample, _) = self.psg.sample();
+                self.audio_resampler.collect_psg_sample(psg_sample);
             }
             self.cycles.decrement_psg();
         }
@@ -245,14 +256,14 @@ impl EmulatorTrait for Sega32XEmulator {
 
         let mut tick_effect = TickEffect::None;
         if self.vdp.tick(mclk_cycles, &mut self.memory) == VdpTickEffect::FrameComplete {
-            self.memory.medium_mut().vdp.composite_frame(
+            self.memory.medium_mut().vdp().composite_frame(
                 self.vdp.frame_size(),
                 self.vdp.border_size(),
                 self.vdp.frame_buffer_mut(),
             );
             self.render_frame(renderer).map_err(Sega32XError::Render)?;
 
-            let cartridge = &mut self.memory.medium_mut().cartridge;
+            let cartridge = self.memory.medium_mut().cartridge_mut();
             if cartridge.persistent_memory_dirty() {
                 cartridge.clear_persistent_dirty_bit();
                 save_writer
@@ -263,8 +274,8 @@ impl EmulatorTrait for Sega32XEmulator {
             tick_effect = TickEffect::FrameRendered;
         }
 
-        debug_assert_eq!(self.vdp.scanline(), self.memory.medium().vdp.scanline());
-        debug_assert_eq!(self.vdp.scanline_mclk(), self.memory.medium().vdp.scanline_mclk());
+        debug_assert_eq!(self.vdp.scanline(), self.memory.medium_mut().vdp().scanline());
+        debug_assert_eq!(self.vdp.scanline_mclk(), self.memory.medium_mut().vdp().scanline_mclk());
 
         Ok(tick_effect)
     }
@@ -302,15 +313,11 @@ impl EmulatorTrait for Sega32XEmulator {
     }
 
     fn hard_reset<S: SaveWriter>(&mut self, save_writer: &mut S) {
-        let rom = mem::take(&mut self.memory.medium_mut().cartridge.rom.0);
+        let rom = mem::take(&mut self.memory.medium_mut().cartridge_mut().rom.0);
 
         *self = Self::create(rom, self.config, save_writer);
     }
 
-    fn timing_mode(&self) -> TimingMode {
-        self.timing_mode
-    }
-
     fn target_fps(&self) -> f64 {
         genesis_core::target_framerate(self.timing_mode)
     }
