@@ -8,28 +8,30 @@ use crate::rf5c164::Rf5c164;
 use bincode::{Decode, Encode};
 use cdrom::CdRomError;
 use cdrom::reader::{CdRom, CdRomFileFormat};
-use genesis_core::input::InputState;
+use genesis_core::input::{GenesisButton, InputState};
 use genesis_core::memory::{MainBus, MainBusSignals, MainBusWrites, Memory};
 use genesis_core::timing::CycleCounters;
 use genesis_core::vdp::{Vdp, VdpTickEffect};
 use genesis_core::ym2612::{Ym2612, YmTickEffect};
-use genesis_core::{GenesisAspectRatio, GenesisEmulatorConfig, GenesisInputs, GenesisRegion};
+use genesis_core::{GenesisEmulatorConfig, GenesisInputs, GenesisRegion};
 use jgenesis_common::frontend::{
-    AudioOutput, Color, EmulatorTrait, PartialClone, Renderer, SaveWriter, TickEffect, TimingMode,
+    AudioOutput, Color, EmulatorConfigTrait, EmulatorTrait, PartialClone, Renderer, SaveWriter,
+    TickEffect, TimingMode,
 };
-use jgenesis_proc_macros::{EnumAll, EnumDisplay, EnumFromStr};
+use jgenesis_proc_macros::{ConfigDisplay, EnumAll, EnumDisplay, EnumFromStr};
 use m68000_emu::M68000;
 use smsgg_core::psg::{Sn76489, Sn76489TickEffect, Sn76489Version};
 use std::fmt::{Debug, Display};
+use std::num::{NonZeroU16, NonZeroU64};
 use std::path::Path;
 use thiserror::Error;
 use z80_emu::Z80;
 
-pub(crate) const SUB_CPU_DIVIDER: u64 = 4;
+pub const DEFAULT_SUB_CPU_DIVIDER: u64 = 4;
 
 const NTSC_GENESIS_MASTER_CLOCK_RATE: u64 = 53_693_175;
 const PAL_GENESIS_MASTER_CLOCK_RATE: u64 = 53_203_424;
-pub(crate) const SEGA_CD_MASTER_CLOCK_RATE: u64 = 50_000_000;
+pub const SEGA_CD_MASTER_CLOCK_RATE: u64 = 50_000_000;
 
 const BIOS_LEN: usize = memory::BIOS_LEN;
 
@@ -76,18 +78,47 @@ pub enum PcmInterpolation {
     None,
     Linear,
     CubicHermite,
+    CubicHermite6Point,
 }
 
-#[derive(Debug, Clone, Copy, Encode, Decode)]
+#[derive(
+    Debug, Clone, Copy, PartialEq, Eq, Default, Encode, Decode, EnumDisplay, EnumFromStr, EnumAll,
+)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "clap", derive(jgenesis_proc_macros::CustomValueEnum))]
+pub enum PcmLowPassFilter {
+    None,
+    #[default]
+    SegaCd,
+}
+
+#[derive(Debug, Clone, Copy, Encode, Decode, ConfigDisplay)]
 pub struct SegaCdEmulatorConfig {
+    #[cfg_display(skip)]
     pub genesis: GenesisEmulatorConfig,
     pub pcm_interpolation: PcmInterpolation,
     pub enable_ram_cartridge: bool,
     pub load_disc_into_ram: bool,
+    pub disc_drive_speed: NonZeroU16,
+    pub sub_cpu_divider: NonZeroU64,
+    pub pcm_low_pass: PcmLowPassFilter,
+    pub apply_genesis_lpf_to_pcm: bool,
+    pub apply_genesis_lpf_to_cd_da: bool,
     pub pcm_enabled: bool,
     pub cd_audio_enabled: bool,
 }
 
+impl EmulatorConfigTrait for SegaCdEmulatorConfig {
+    fn with_overclocking_disabled(&self) -> Self {
+        Self {
+            genesis: self.genesis.with_overclocking_disabled(),
+            disc_drive_speed: NonZeroU16::new(1).unwrap(),
+            sub_cpu_divider: NonZeroU64::new(DEFAULT_SUB_CPU_DIVIDER).unwrap(),
+            ..*self
+        }
+    }
+}
+
 #[derive(Debug, Encode, Decode, PartialClone)]
 pub struct SegaCdEmulator {
     #[partial_clone(partial)]
@@ -104,15 +135,13 @@ pub struct SegaCdEmulator {
     audio_resampler: AudioResampler,
     timing_mode: TimingMode,
     main_bus_writes: MainBusWrites,
-    aspect_ratio: GenesisAspectRatio,
-    adjust_aspect_ratio_in_2x_resolution: bool,
     disc_title: String,
     cycles: SegaCdCycleCounters,
     sega_cd_mclk_cycles: u64,
     sega_cd_mclk_cycle_product: u64,
+    sub_cpu_divider: u64,
     sub_cpu_wait_cycles: u64,
     sub_cpu_pending_intack: Option<u8>,
-    load_disc_into_ram: bool,
     config: SegaCdEmulatorConfig,
 }
 
@@ -193,14 +222,8 @@ impl SegaCdEmulator {
 
         let initial_backup_ram = save_writer.load_bytes("sav").ok();
         let initial_ram_cartridge = save_writer.load_bytes("ramc").ok();
-        let mut sega_cd = SegaCd::new(
-            bios,
-            disc,
-            initial_backup_ram,
-            initial_ram_cartridge,
-            emulator_config.enable_ram_cartridge,
-            emulator_config.genesis.forced_region,
-        )?;
+        let mut sega_cd =
+            SegaCd::new(bios, disc, initial_backup_ram, initial_ram_cartridge, &emulator_config)?;
         let disc_title = sega_cd.disc_title()?.unwrap_or("(no disc)".into());
 
         let memory = Memory::new(sega_cd);
@@ -242,17 +265,13 @@ impl SegaCdEmulator {
             audio_resampler,
             timing_mode,
             main_bus_writes: MainBusWrites::new(),
-            aspect_ratio: emulator_config.genesis.aspect_ratio,
-            adjust_aspect_ratio_in_2x_resolution: emulator_config
-                .genesis
-                .adjust_aspect_ratio_in_2x_resolution,
             disc_title,
             cycles: SegaCdCycleCounters::new(emulator_config.genesis.clamped_m68k_divider()),
             sega_cd_mclk_cycles: 0,
             sega_cd_mclk_cycle_product: 0,
+            sub_cpu_divider: emulator_config.sub_cpu_divider.get(),
             sub_cpu_wait_cycles: 0,
             sub_cpu_pending_intack: None,
-            load_disc_into_ram: emulator_config.load_disc_into_ram,
             config: emulator_config,
         };
 
@@ -290,8 +309,8 @@ impl SegaCdEmulator {
     fn render_frame<R: Renderer>(&self, renderer: &mut R) -> Result<(), R::Err> {
         genesis_core::render_frame(
             &self.vdp,
-            self.aspect_ratio,
-            self.adjust_aspect_ratio_in_2x_resolution,
+            self.config.genesis.aspect_ratio,
+            self.config.genesis.adjust_aspect_ratio_in_2x_resolution,
             renderer,
         )
     }
@@ -315,7 +334,7 @@ impl SegaCdEmulator {
         format: CdRomFileFormat,
     ) -> SegaCdLoadResult<()> {
         let sega_cd = self.memory.medium_mut();
-        sega_cd.change_disc(rom_path, format, self.load_disc_into_ram)?;
+        sega_cd.change_disc(rom_path, format, self.config.load_disc_into_ram)?;
         self.disc_title = sega_cd.disc_title()?.unwrap_or_else(|| "(no disc)".into());
 
         Ok(())
@@ -335,6 +354,7 @@ impl SegaCdEmulator {
 }
 
 impl EmulatorTrait for SegaCdEmulator {
+    type Button = GenesisButton;
     type Inputs = GenesisInputs;
     type Config = SegaCdEmulatorConfig;
 
@@ -401,10 +421,24 @@ impl EmulatorTrait for SegaCdEmulator {
         let prev_scd_mclk_cycles = self.sega_cd_mclk_cycles;
         self.sega_cd_mclk_cycles += scd_mclk_elapsed;
 
-        let sub_cpu_cycles =
-            self.sega_cd_mclk_cycles / SUB_CPU_DIVIDER - prev_scd_mclk_cycles / SUB_CPU_DIVIDER;
+        let pcm_cycles = self.sega_cd_mclk_cycles / DEFAULT_SUB_CPU_DIVIDER
+            - prev_scd_mclk_cycles / DEFAULT_SUB_CPU_DIVIDER;
         let elapsed_scd_mclk_cycles = self.sega_cd_mclk_cycles - prev_scd_mclk_cycles;
 
+        // This match seems silly, but it avoids doing an integer division for the common dividers
+        // of 1-4. Dividers higher than 4 can only be set via the CLI or by manually editing config
+        // (and underclocking probably won't work well anyway)
+        let sub_cpu_cycles = match self.sub_cpu_divider {
+            DEFAULT_SUB_CPU_DIVIDER => pcm_cycles,
+            3 => self.sega_cd_mclk_cycles / 3 - prev_scd_mclk_cycles / 3,
+            2 => (self.sega_cd_mclk_cycles >> 1) - (prev_scd_mclk_cycles >> 1),
+            1 => elapsed_scd_mclk_cycles,
+            _ => {
+                self.sega_cd_mclk_cycles / self.sub_cpu_divider
+                    - prev_scd_mclk_cycles / self.sub_cpu_divider
+            }
+        };
+
         // Disc drive and timer/stopwatch
         let sega_cd = self.memory.medium_mut();
         sega_cd.tick(elapsed_scd_mclk_cycles, &mut self.pcm, |sample_l, sample_r| {
@@ -433,8 +467,9 @@ impl EmulatorTrait for SegaCdEmulator {
         // PSG
         while self.cycles.should_tick_psg() {
             if self.psg.tick() == Sn76489TickEffect::Clocked {
-                let (psg_sample_l, psg_sample_r) = self.psg.sample();
-                self.audio_resampler.collect_psg_sample(psg_sample_l, psg_sample_r);
+                // PSG output is mono in Genesis; stereo output is only for Game Gear
+                let (psg_sample, _) = self.psg.sample();
+                self.audio_resampler.collect_psg_sample(psg_sample);
             }
             self.cycles.decrement_psg();
         }
@@ -449,7 +484,7 @@ impl EmulatorTrait for SegaCdEmulator {
         }
 
         // RF5C164
-        self.pcm.tick(sub_cpu_cycles, |(pcm_sample_l, pcm_sample_r)| {
+        self.pcm.tick(pcm_cycles, |(pcm_sample_l, pcm_sample_r)| {
             self.audio_resampler.collect_pcm_sample(pcm_sample_l, pcm_sample_r);
         });
 
@@ -491,19 +526,16 @@ impl EmulatorTrait for SegaCdEmulator {
     }
 
     fn reload_config(&mut self, config: &Self::Config) {
-        self.aspect_ratio = config.genesis.aspect_ratio;
-        self.adjust_aspect_ratio_in_2x_resolution =
-            config.genesis.adjust_aspect_ratio_in_2x_resolution;
         self.vdp.reload_config(config.genesis.to_vdp_config());
         self.ym2612.reload_config(config.genesis);
         self.pcm.reload_config(config);
         self.input.reload_config(config.genesis);
         self.audio_resampler.reload_config(*config);
         self.cycles.update_m68k_divider(config.genesis.clamped_m68k_divider());
+        self.sub_cpu_divider = config.sub_cpu_divider.get();
 
         let sega_cd = self.memory.medium_mut();
-        sega_cd.set_forced_region(config.genesis.forced_region);
-        sega_cd.set_enable_ram_cartridge(config.enable_ram_cartridge);
+        sega_cd.reload_config(config);
 
         self.config = *config;
     }
@@ -532,10 +564,6 @@ impl EmulatorTrait for SegaCdEmulator {
             .expect("Hard reset should not cause an I/O error");
     }
 
-    fn timing_mode(&self) -> TimingMode {
-        self.timing_mode
-    }
-
     fn target_fps(&self) -> f64 {
         genesis_core::target_framerate(self.timing_mode)
     }
