@@ -3,7 +3,7 @@
 use crate::apu::{Apu, ApuTickEffect};
 use crate::audio::AudioResampler;
 use crate::bus::Bus;
-use crate::input::SnesInputs;
+use crate::input::{SnesButton, SnesInputs};
 use crate::memory::dma::{DmaStatus, DmaUnit};
 use crate::memory::{CpuInternalRegisters, Memory};
 use crate::ppu::{Ppu, PpuTickEffect};
@@ -11,10 +11,12 @@ use bincode::error::EncodeError;
 use bincode::{Decode, Encode};
 use crc::Crc;
 use jgenesis_common::frontend::{
-    AudioOutput, Color, EmulatorTrait, FrameSize, PartialClone, PixelAspectRatio, Renderer,
-    SaveWriter, TickEffect, TimingMode,
+    AudioOutput, Color, EmulatorConfigTrait, EmulatorTrait, FrameSize, PartialClone,
+    PixelAspectRatio, Renderer, SaveWriter, TickEffect, TimingMode,
+};
+use jgenesis_proc_macros::{
+    ConfigDisplay, EnumAll, EnumDisplay, EnumFromStr, FakeDecode, FakeEncode,
 };
-use jgenesis_proc_macros::{EnumAll, EnumDisplay, EnumFromStr, FakeDecode, FakeEncode};
 use std::fmt::{Debug, Display};
 use std::num::NonZeroU64;
 use std::{io, mem};
@@ -76,15 +78,18 @@ pub enum AudioInterpolationMode {
     Hermite,
 }
 
-#[derive(Debug, Clone, Copy, Encode, Decode)]
+#[derive(Debug, Clone, Copy, Encode, Decode, ConfigDisplay)]
 pub struct SnesEmulatorConfig {
     pub forced_timing_mode: Option<TimingMode>,
     pub aspect_ratio: SnesAspectRatio,
+    pub deinterlace: bool,
     pub audio_interpolation: AudioInterpolationMode,
     pub audio_60hz_hack: bool,
     pub gsu_overclock_factor: NonZeroU64,
 }
 
+impl EmulatorConfigTrait for SnesEmulatorConfig {}
+
 pub type CoprocessorRomFn = dyn Fn() -> Result<Vec<u8>, (io::Error, String)>;
 
 #[derive(Default, FakeEncode, FakeDecode)]
@@ -210,7 +215,7 @@ impl SnesEmulator {
 
         let timing_mode =
             config.forced_timing_mode.unwrap_or_else(|| memory.cartridge_timing_mode());
-        let ppu = Ppu::new(timing_mode);
+        let ppu = Ppu::new(timing_mode, config);
         let apu = Apu::new(timing_mode, config);
 
         log::info!("Running with timing/display mode {timing_mode}");
@@ -273,6 +278,7 @@ impl SnesEmulator {
 }
 
 impl EmulatorTrait for SnesEmulator {
+    type Button = SnesButton;
     type Inputs = SnesInputs;
     type Config = SnesEmulatorConfig;
 
@@ -407,6 +413,7 @@ impl EmulatorTrait for SnesEmulator {
 
     fn reload_config(&mut self, config: &Self::Config) {
         self.aspect_ratio = config.aspect_ratio;
+        self.ppu.update_config(*config);
         self.apu.update_config(*config);
         self.memory.update_gsu_overclock_factor(config.gsu_overclock_factor);
 
@@ -441,10 +448,6 @@ impl EmulatorTrait for SnesEmulator {
             .expect("Hard resetting should never fail to load");
     }
 
-    fn timing_mode(&self) -> TimingMode {
-        self.timing_mode
-    }
-
     fn target_fps(&self) -> f64 {
         match (self.timing_mode, self.emulator_config.audio_60hz_hack) {
             (TimingMode::Ntsc, true) => 60.0,
