@@ -181,7 +181,7 @@ impl Cartridge {
         });
 
         if cartridge_type != CartridgeType::Spc7110 {
-            mirror_to_next_power_of_two(&mut rom);
+            jgenesis_common::rom::mirror_to_next_power_of_two(&mut rom);
             log::debug!("ROM length after mirroring is {} bytes", rom.len());
 
             debug_assert_eq!(rom.len().count_ones(), 1);
@@ -217,7 +217,7 @@ impl Cartridge {
             upd77c25::ST01X_RAM_LEN_BYTES
         } else if cartridge_type == CartridgeType::SuperFx {
             superfx::guess_ram_len(&rom)
-        } else if sram_header_byte == 0 {
+        } else if sram_header_byte == 0 || sram_header_byte > 21 {
             0
         } else {
             1 << (10 + sram_header_byte)
@@ -887,82 +887,3 @@ fn exhirom_map_rom_address(address: u32, rom_len: u32) -> u32 {
     let rom_addr = (address & 0x3FFFFF) | (((address >> 1) & 0x400000) ^ 0x400000);
     rom_addr & (rom_len - 1)
 }
-
-fn mirror_to_next_power_of_two(rom: &mut Vec<u8>) {
-    if rom.is_empty() {
-        log::error!("Cannot mirror empty ROM");
-        return;
-    }
-
-    let ones_count = rom.len().count_ones();
-    if ones_count == 1 {
-        // ROM size is already a power of two
-        return;
-    }
-
-    let trailing_zeroes = rom.len().trailing_zeros();
-    let source_len = 1 << trailing_zeroes;
-    let source_mask = source_len - 1;
-
-    let remaining_rom_len = rom.len() & !source_len;
-    let copy_len = (1 << (remaining_rom_len.trailing_zeros())) - source_len;
-
-    log::debug!(
-        "ROM len is {}; duplicating last {source_len} bytes of ROM to last {copy_len} bytes",
-        rom.len()
-    );
-
-    let base_addr = rom.len() & !source_len;
-    for i in 0..copy_len {
-        rom.push(rom[base_addr + (i & source_mask)]);
-    }
-
-    // Recurse in case there are more than 2 ROM chips (e.g. fan translated version of Daikaijuu
-    // Monogatari II which is 5.5MB)
-    mirror_to_next_power_of_two(rom);
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use std::array;
-
-    fn new_vec<const LEN: usize>() -> Vec<u8> {
-        Vec::from(array::from_fn::<u8, LEN, _>(|i| i as u8))
-    }
-
-    #[test]
-    fn mirror_empty_rom() {
-        let mut rom = vec![];
-        mirror_to_next_power_of_two(&mut rom);
-        assert_eq!(rom, vec![]);
-    }
-
-    #[test]
-    fn mirror_power_of_two() {
-        let mut rom = new_vec::<8>();
-        mirror_to_next_power_of_two(&mut rom);
-        assert_eq!(rom, vec![0, 1, 2, 3, 4, 5, 6, 7]);
-    }
-
-    #[test]
-    fn mirror_6_to_8() {
-        let mut rom = new_vec::<6>();
-        mirror_to_next_power_of_two(&mut rom);
-        assert_eq!(rom, vec![0, 1, 2, 3, 4, 5, 4, 5]);
-    }
-
-    #[test]
-    fn mirror_5_to_8() {
-        let mut rom = new_vec::<5>();
-        mirror_to_next_power_of_two(&mut rom);
-        assert_eq!(rom, vec![0, 1, 2, 3, 4, 4, 4, 4]);
-    }
-
-    #[test]
-    fn mirror_11_to_16() {
-        let mut rom = new_vec::<11>();
-        mirror_to_next_power_of_two(&mut rom);
-        assert_eq!(rom, vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 10, 8, 9, 10, 10]);
-    }
-}
