@@ -2,6 +2,7 @@ use bincode::{Decode, Encode};
 use jgenesis_proc_macros::{EnumAll, EnumDisplay, EnumFromStr};
 use std::error::Error;
 use std::fmt::{Debug, Display};
+use std::hash::Hash;
 use std::num::NonZeroU32;
 
 #[repr(C)]
@@ -44,6 +45,14 @@ pub struct FrameSize {
     pub height: u32,
 }
 
+#[derive(Debug, Clone, Copy)]
+pub struct DisplayArea {
+    pub width: u32,
+    pub height: u32,
+    pub x: u32,
+    pub y: u32,
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Encode, Decode)]
 pub struct PixelAspectRatio(f64);
 
@@ -155,6 +164,7 @@ pub trait PartialClone {
     fn partial_clone(&self) -> Self;
 }
 
+use crate::input::Player;
 pub use jgenesis_proc_macros::PartialClone;
 
 #[derive(
@@ -176,9 +186,33 @@ pub enum TickEffect {
 
 pub type TickResult<Err> = Result<TickEffect, Err>;
 
+pub trait MappableInputs<Button> {
+    fn set_field(&mut self, button: Button, player: Player, pressed: bool);
+
+    #[allow(unused_variables)]
+    fn handle_mouse_motion(
+        &mut self,
+        x: i32,
+        y: i32,
+        frame_size: FrameSize,
+        display_area: DisplayArea,
+    ) {
+    }
+
+    fn handle_mouse_leave(&mut self) {}
+}
+
+pub trait EmulatorConfigTrait: Clone {
+    #[must_use]
+    fn with_overclocking_disabled(&self) -> Self {
+        self.clone()
+    }
+}
+
 pub trait EmulatorTrait: Encode + Decode + PartialClone {
-    type Inputs;
-    type Config;
+    type Button: Debug + Copy + Eq + Hash;
+    type Inputs: Default + MappableInputs<Self::Button>;
+    type Config: EmulatorConfigTrait;
 
     type Err<RErr: Debug + Display + Send + Sync + 'static, AErr: Debug + Display + Send + Sync + 'static, SErr: Debug + Display + Send + Sync + 'static>: Error + Send + Sync + 'static;
 
@@ -221,7 +255,11 @@ pub trait EmulatorTrait: Encode + Decode + PartialClone {
 
     fn hard_reset<S: SaveWriter>(&mut self, save_writer: &mut S);
 
-    fn timing_mode(&self) -> TimingMode;
+    // All cores start at save state version 2; they can override this function when they need to change it
+    #[must_use]
+    fn save_state_version() -> u16 {
+        2
+    }
 
     fn target_fps(&self) -> f64;
 
