@@ -293,12 +293,11 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
             0x42 | 0x52 | 0x62 | 0x72 => self.sbc_hl_ss(opcode2),
             0x43 | 0x53 | 0x63 | 0x73 => self.ld_direct_dd(opcode2),
             0x44 => self.neg(),
-            0x45 => self.retn(),
+            0x45 | 0x4D => self.reti_retn(),
             0x46 => self.im(InterruptMode::Mode0),
             0x47 => self.ld_ir_a(Register8::I),
             0x4A | 0x5A | 0x6A | 0x7A => self.adc_hl_ss(opcode2),
             0x4B | 0x5B | 0x6B | 0x7B => self.ld_dd_direct(opcode2),
-            0x4D => self.reti(),
             0x4F => self.ld_ir_a(Register8::R),
             0x56 => self.im(InterruptMode::Mode1),
             0x57 => self.ld_a_ir(Register8::I),
@@ -348,7 +347,7 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
         let ParseResult { opcode, index_prefix: index, index_fetch_t_cycles } = self.parse_opcode();
 
         log::trace!(
-            "PC={:04X}, opcode={opcode:02X} ({}), index={index:?}, a={:02X}, next={:02X} {:02X}, sp={:04X}, a={:02X}, f={:02X}, b={:02X}, c={:02X}, d={:02X}, e={:02X}, h={:02X}, l={:02X}, iff1={}",
+            "PC={:04X}, opcode={opcode:02X} ({}), index={index:?}, a={:02X}, next={:02X} {:02X}, sp={:04X}, a={:02X}, f={:02X}, b={:02X}, c={:02X}, d={:02X}, e={:02X}, h={:02X}, l={:02X}, ix={:04X}, iy={:04X}, iff1={}",
             self.registers.pc.wrapping_sub(1),
             mnemonics::for_opcode(opcode, self.bus.read_memory(self.registers.pc)),
             self.registers.a,
@@ -363,6 +362,8 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
             self.registers.e,
             self.registers.h,
             self.registers.l,
+            self.registers.ix,
+            self.registers.iy,
             self.registers.iff1
         );
 
