@@ -7,12 +7,13 @@ mod romlist;
 mod smsgg;
 mod snes;
 
-use crate::app::common::SavePathSelect;
 use crate::app::input::{GenericButton, InputMappingSet};
 use crate::app::nes::OverscanState;
 use crate::app::romlist::{RomListThreadHandle, RomMetadata};
+use crate::app::snes::HandledError;
 use crate::emuthread;
 use crate::emuthread::{EmuThreadCommand, EmuThreadHandle, EmuThreadStatus};
+use crate::widgets::SavePathSelect;
 use eframe::Frame;
 use egui::panel::TopBottomSide;
 use egui::{
@@ -23,6 +24,7 @@ use egui::{
 use egui_extras::{Column, TableBuilder};
 use jgenesis_native_config::{AppConfig, EguiTheme, ListFilters, RecentOpen};
 use jgenesis_native_driver::config::HideMouseCursor;
+use jgenesis_native_driver::{NativeEmulatorError, extensions};
 use jgenesis_proc_macros::{EnumAll, EnumDisplay, EnumFromStr};
 use jgenesis_renderer::config::Scanlines;
 use rfd::FileDialog;
@@ -32,8 +34,6 @@ use std::path::{Path, PathBuf};
 use std::rc::Rc;
 use std::str::FromStr;
 use std::sync::{Arc, Mutex};
-use time::util::local_offset;
-use time::util::local_offset::Soundness;
 use time::{OffsetDateTime, UtcOffset, format_description};
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, EnumAll, EnumDisplay, EnumFromStr)]
@@ -116,6 +116,7 @@ enum OpenWindow {
     GenesisAudio,
     NesAudio,
     SnesAudio,
+    GameBoyAudio,
     GeneralInput,
     SmsGgInput,
     GenesisInput,
@@ -135,7 +136,7 @@ struct HelpText {
 }
 
 struct AppState {
-    current_file_path: String,
+    current_file_path: PathBuf,
     open_windows: HashSet<OpenWindow>,
     help_text: HashMap<OpenWindow, HelpText>,
     input_mapping_sets: HashMap<OpenWindow, InputMappingSet>,
@@ -168,7 +169,7 @@ impl AppState {
     fn from_config(config: &AppConfig) -> Self {
         let recent_open_list = romlist::from_recent_opens(&config.recent_open_list);
         Self {
-            current_file_path: String::new(),
+            current_file_path: PathBuf::new(),
             open_windows: HashSet::new(),
             help_text: HashMap::new(),
             input_mapping_sets: HashMap::new(),
@@ -256,7 +257,7 @@ impl<T: Copy + FromStr> Widget for NumericTextEdit<'_, T> {
 
 #[derive(Debug, Clone)]
 pub struct LoadAtStartup {
-    pub file_path: String,
+    pub file_path: PathBuf,
     pub load_state_slot: Option<usize>,
 }
 
@@ -316,7 +317,7 @@ impl App {
                     console.supported_extensions().iter().copied().chain(["zip", "7z"]).collect();
                 file_dialog.add_filter(console.display_str(), &extensions)
             }
-            None => file_dialog.add_filter("Supported Files", romlist::ALL_EXTENSIONS),
+            None => file_dialog.add_filter("Supported Files", &extensions::ALL_PLUS_ARCHIVES),
         };
 
         file_dialog = file_dialog.add_filter("All Files", &["*"]);
@@ -326,18 +327,17 @@ impl App {
         }
         let Some(path) = file_dialog.pick_file() else { return };
 
-        let Some(path_str) = path.to_str().map(String::from) else { return };
-        self.launch_emulator(path_str, console);
+        self.launch_emulator(path, console);
     }
 
-    fn launch_emulator(&mut self, path: String, console: Option<Console>) {
+    fn launch_emulator(&mut self, path: PathBuf, console: Option<Console>) {
         self.state.current_file_path.clone_from(&path);
 
         let console = match console {
             Some(console) => console,
             None => {
                 let Some(metadata) = romlist::read_metadata(Path::new(&path)) else {
-                    log::error!("Unable to detect compatible file at path: '{path}'");
+                    log::error!("Unable to detect compatible file at path: '{}'", path.display());
                     self.emu_thread.clear_waiting_for_first_command();
                     return;
                 };
@@ -686,7 +686,7 @@ impl App {
                 ui.close_menu();
             }
 
-            if ui.button("Genesis / Sega CD").clicked() {
+            if ui.button("Genesis / Sega CD / 32X").clicked() {
                 self.state.open_windows.insert(OpenWindow::GenesisGeneral);
                 ui.close_menu();
             }
@@ -739,7 +739,7 @@ impl App {
                 ui.close_menu();
             }
 
-            if ui.button("Genesis / Sega CD").clicked() {
+            if ui.button("Genesis / Sega CD / 32X").clicked() {
                 self.state.open_windows.insert(OpenWindow::GenesisVideo);
                 ui.close_menu();
             }
@@ -775,7 +775,7 @@ impl App {
                 ui.close_menu();
             }
 
-            if ui.button("Genesis / Sega CD").clicked() {
+            if ui.button("Genesis / Sega CD / 32X").clicked() {
                 self.state.open_windows.insert(OpenWindow::GenesisAudio);
                 ui.close_menu();
             }
@@ -789,6 +789,11 @@ impl App {
                 self.state.open_windows.insert(OpenWindow::SnesAudio);
                 ui.close_menu();
             }
+
+            if ui.button("Game Boy").clicked() {
+                self.state.open_windows.insert(OpenWindow::GameBoyAudio);
+                ui.close_menu();
+            }
         });
     }
 
@@ -806,7 +811,7 @@ impl App {
                 ui.close_menu();
             }
 
-            if ui.button("Genesis / Sega CD").clicked() {
+            if ui.button("Genesis / Sega CD / 32X").clicked() {
                 self.state.open_windows.insert(OpenWindow::GenesisInput);
                 ui.close_menu();
             }
@@ -988,6 +993,44 @@ impl App {
         });
     }
 
+    fn render_windows(&mut self, ctx: &Context) {
+        let open_windows: Vec<_> = self.state.open_windows.iter().copied().collect();
+        for open_window in open_windows {
+            match open_window {
+                OpenWindow::SmsGgGeneral => self.render_smsgg_general_settings(ctx),
+                OpenWindow::GenesisGeneral => self.render_genesis_general_settings(ctx),
+                OpenWindow::NesGeneral => self.render_nes_general_settings(ctx),
+                OpenWindow::SnesGeneral => self.render_snes_general_settings(ctx),
+                OpenWindow::GameBoyGeneral => self.render_gb_general_settings(ctx),
+                OpenWindow::Synchronization => self.render_sync_settings(ctx),
+                OpenWindow::Paths => self.render_path_settings(ctx),
+                OpenWindow::Interface => self.render_interface_settings(ctx),
+                OpenWindow::CommonVideo => self.render_common_video_settings(ctx),
+                OpenWindow::SmsGgVideo => self.render_smsgg_video_settings(ctx),
+                OpenWindow::GenesisVideo => self.render_genesis_video_settings(ctx),
+                OpenWindow::NesVideo => self.render_nes_video_settings(ctx),
+                OpenWindow::SnesVideo => self.render_snes_video_settings(ctx),
+                OpenWindow::GameBoyVideo => self.render_gb_video_settings(ctx),
+                OpenWindow::CommonAudio => self.render_common_audio_settings(ctx),
+                OpenWindow::SmsGgAudio => self.render_smsgg_audio_settings(ctx),
+                OpenWindow::GenesisAudio => self.render_genesis_audio_settings(ctx),
+                OpenWindow::NesAudio => self.render_nes_audio_settings(ctx),
+                OpenWindow::SnesAudio => self.render_snes_audio_settings(ctx),
+                OpenWindow::GameBoyAudio => self.render_gb_audio_settings(ctx),
+                OpenWindow::GeneralInput => self.render_general_input_settings(ctx),
+                OpenWindow::SmsGgInput => self.render_smsgg_input_settings(ctx),
+                OpenWindow::GenesisInput => self.render_genesis_input_settings(ctx),
+                OpenWindow::NesInput => self.render_nes_input_settings(ctx),
+                OpenWindow::NesPeripherals => self.render_nes_peripheral_settings(ctx),
+                OpenWindow::SnesInput => self.render_snes_input_settings(ctx),
+                OpenWindow::SnesPeripherals => self.render_snes_peripheral_settings(ctx),
+                OpenWindow::GameBoyInput => self.render_gb_input_settings(ctx),
+                OpenWindow::Hotkeys => self.render_hotkey_settings(ctx),
+                OpenWindow::About => self.render_about(ctx),
+            }
+        }
+    }
+
     fn render_help_text(&mut self, ui: &mut Ui, window: OpenWindow) {
         let Some(help_text) = self.state.help_text.get(&window) else { return };
 
@@ -1002,20 +1045,36 @@ impl App {
     }
 
     fn check_emulator_error(&mut self, ctx: &Context) {
-        let mut error_lock = self.emu_thread.lock_emulator_error();
+        let emulator_error = self.emu_thread.emulator_error();
+        let mut error_lock = emulator_error.lock().unwrap();
         self.state.error_window_open = error_lock.is_some();
 
-        if let Some(error) = error_lock.as_ref() {
+        if let Some(err) = error_lock.as_ref() {
             let mut open = true;
-            Window::new("Emulator Error").open(&mut open).resizable(false).show(ctx, |ui| {
-                ui.colored_label(Color32::RED, format!("Emulator terminated with error: {error}"));
-            });
+            match err {
+                NativeEmulatorError::SegaCdNoBios => self.render_scd_bios_error(ctx, &mut open),
+                NativeEmulatorError::SnesLoad(snes_load_err) => {
+                    match self.render_snes_load_error(ctx, snes_load_err, &mut open) {
+                        HandledError::Yes => {}
+                        HandledError::No => Self::render_generic_error_window(ctx, err, &mut open),
+                    }
+                }
+                _ => Self::render_generic_error_window(ctx, err, &mut open),
+            }
             if !open {
                 *error_lock = None;
             }
         }
     }
 
+    fn render_generic_error_window(ctx: &Context, err: &NativeEmulatorError, open: &mut bool) {
+        Window::new("Emulator Error").open(open).resizable(false).show(ctx, |ui| {
+            ui.label("Emulator terminated with error:");
+            ui.add_space(10.0);
+            ui.colored_label(Color32::RED, err.to_string());
+        });
+    }
+
     fn check_waiting_for_input(&mut self, ctx: &Context) {
         if let Some((button, mapping)) = self.state.waiting_for_input {
             if let Ok(input) = self.emu_thread.poll_input_receiver() {
@@ -1107,39 +1166,7 @@ impl eframe::App for App {
         self.render_menu(ctx);
         self.render_central_panel(ctx);
 
-        for open_window in self.state.open_windows.clone() {
-            match open_window {
-                OpenWindow::SmsGgGeneral => self.render_smsgg_general_settings(ctx),
-                OpenWindow::GenesisGeneral => self.render_genesis_general_settings(ctx),
-                OpenWindow::NesGeneral => self.render_nes_general_settings(ctx),
-                OpenWindow::SnesGeneral => self.render_snes_general_settings(ctx),
-                OpenWindow::GameBoyGeneral => self.render_gb_general_settings(ctx),
-                OpenWindow::Synchronization => self.render_sync_settings(ctx),
-                OpenWindow::Paths => self.render_path_settings(ctx),
-                OpenWindow::Interface => self.render_interface_settings(ctx),
-                OpenWindow::CommonVideo => self.render_common_video_settings(ctx),
-                OpenWindow::SmsGgVideo => self.render_smsgg_video_settings(ctx),
-                OpenWindow::GenesisVideo => self.render_genesis_video_settings(ctx),
-                OpenWindow::NesVideo => self.render_nes_video_settings(ctx),
-                OpenWindow::SnesVideo => self.render_snes_video_settings(ctx),
-                OpenWindow::GameBoyVideo => self.render_gb_video_settings(ctx),
-                OpenWindow::CommonAudio => self.render_common_audio_settings(ctx),
-                OpenWindow::SmsGgAudio => self.render_smsgg_audio_settings(ctx),
-                OpenWindow::GenesisAudio => self.render_genesis_audio_settings(ctx),
-                OpenWindow::NesAudio => self.render_nes_audio_settings(ctx),
-                OpenWindow::SnesAudio => self.render_snes_audio_settings(ctx),
-                OpenWindow::GeneralInput => self.render_general_input_settings(ctx),
-                OpenWindow::SmsGgInput => self.render_smsgg_input_settings(ctx),
-                OpenWindow::GenesisInput => self.render_genesis_input_settings(ctx),
-                OpenWindow::NesInput => self.render_nes_input_settings(ctx),
-                OpenWindow::NesPeripherals => self.render_nes_peripheral_settings(ctx),
-                OpenWindow::SnesInput => self.render_snes_input_settings(ctx),
-                OpenWindow::SnesPeripherals => self.render_snes_peripheral_settings(ctx),
-                OpenWindow::GameBoyInput => self.render_gb_input_settings(ctx),
-                OpenWindow::Hotkeys => self.render_hotkey_settings(ctx),
-                OpenWindow::About => self.render_about(ctx),
-            }
-        }
+        self.render_windows(ctx);
 
         if prev_config != self.config {
             self.state.display_scanlines_warning = should_display_scanlines_warning(&self.config);
@@ -1165,6 +1192,7 @@ fn should_reload_config(prev_config: &AppConfig, new_config: &AppConfig) -> bool
         list_filters: ListFilters::default(),
         rom_search_dirs: vec![],
         recent_open_list: vec![],
+        egui_theme: EguiTheme::default(),
         ..prev_config.clone()
     };
 
@@ -1172,6 +1200,7 @@ fn should_reload_config(prev_config: &AppConfig, new_config: &AppConfig) -> bool
         list_filters: ListFilters::default(),
         rom_search_dirs: vec![],
         recent_open_list: vec![],
+        egui_theme: EguiTheme::default(),
         ..new_config.clone()
     };
 
@@ -1182,15 +1211,7 @@ fn format_time_nanos(time_nanos: u128) -> Option<String> {
     let utc_date_time = OffsetDateTime::from_unix_timestamp_nanos(time_nanos as i128)
         .unwrap_or(OffsetDateTime::UNIX_EPOCH);
 
-    // SAFETY: Nothing in this application modifies the current local time zone offset
-    let local_offset = unsafe {
-        local_offset::set_soundness(Soundness::Unsound);
-        let offset = UtcOffset::current_local_offset().ok();
-        local_offset::set_soundness(Soundness::Sound);
-
-        offset
-    }?;
-
+    let local_offset = UtcOffset::current_local_offset().ok()?;
     let local_date_time = utc_date_time.checked_to_offset(local_offset)?;
 
     let format =
