@@ -219,6 +219,7 @@ fn hotkey_label(hotkey: Hotkey) -> &'static str {
         StepFrame => "Step to next frame:",
         FastForward => "Fast forward:",
         Rewind => "Rewind:",
+        ToggleOverclocking => "Toggle overclocking enabled:",
         OpenDebugger => "Open memory viewer:",
         SaveStateSlot0 => "Save state to slot 0:",
         SaveStateSlot1 => "Save state to slot 1:",
@@ -399,43 +400,46 @@ fn access_hotkey(
     hotkey: Hotkey,
     config: &mut InputAppConfig,
 ) -> &mut Option<Vec<GenericInput>> {
+    use Hotkey::*;
+
     let mapping_config = mapping.hotkey(config);
 
     match hotkey {
-        Hotkey::PowerOff => &mut mapping_config.power_off,
-        Hotkey::Exit => &mut mapping_config.exit,
-        Hotkey::ToggleFullscreen => &mut mapping_config.toggle_fullscreen,
-        Hotkey::SaveState => &mut mapping_config.save_state,
-        Hotkey::LoadState => &mut mapping_config.load_state,
-        Hotkey::NextSaveStateSlot => &mut mapping_config.next_save_state_slot,
-        Hotkey::PrevSaveStateSlot => &mut mapping_config.prev_save_state_slot,
-        Hotkey::SoftReset => &mut mapping_config.soft_reset,
-        Hotkey::HardReset => &mut mapping_config.hard_reset,
-        Hotkey::Pause => &mut mapping_config.pause,
-        Hotkey::StepFrame => &mut mapping_config.step_frame,
-        Hotkey::FastForward => &mut mapping_config.fast_forward,
-        Hotkey::Rewind => &mut mapping_config.rewind,
-        Hotkey::OpenDebugger => &mut mapping_config.open_debugger,
-        Hotkey::SaveStateSlot0 => &mut mapping_config.save_state_slot_0,
-        Hotkey::SaveStateSlot1 => &mut mapping_config.save_state_slot_1,
-        Hotkey::SaveStateSlot2 => &mut mapping_config.save_state_slot_2,
-        Hotkey::SaveStateSlot3 => &mut mapping_config.save_state_slot_3,
-        Hotkey::SaveStateSlot4 => &mut mapping_config.save_state_slot_4,
-        Hotkey::SaveStateSlot5 => &mut mapping_config.save_state_slot_5,
-        Hotkey::SaveStateSlot6 => &mut mapping_config.save_state_slot_6,
-        Hotkey::SaveStateSlot7 => &mut mapping_config.save_state_slot_7,
-        Hotkey::SaveStateSlot8 => &mut mapping_config.save_state_slot_8,
-        Hotkey::SaveStateSlot9 => &mut mapping_config.save_state_slot_9,
-        Hotkey::LoadStateSlot0 => &mut mapping_config.load_state_slot_0,
-        Hotkey::LoadStateSlot1 => &mut mapping_config.load_state_slot_1,
-        Hotkey::LoadStateSlot2 => &mut mapping_config.load_state_slot_2,
-        Hotkey::LoadStateSlot3 => &mut mapping_config.load_state_slot_3,
-        Hotkey::LoadStateSlot4 => &mut mapping_config.load_state_slot_4,
-        Hotkey::LoadStateSlot5 => &mut mapping_config.load_state_slot_5,
-        Hotkey::LoadStateSlot6 => &mut mapping_config.load_state_slot_6,
-        Hotkey::LoadStateSlot7 => &mut mapping_config.load_state_slot_7,
-        Hotkey::LoadStateSlot8 => &mut mapping_config.load_state_slot_8,
-        Hotkey::LoadStateSlot9 => &mut mapping_config.load_state_slot_9,
+        PowerOff => &mut mapping_config.power_off,
+        Exit => &mut mapping_config.exit,
+        ToggleFullscreen => &mut mapping_config.toggle_fullscreen,
+        SaveState => &mut mapping_config.save_state,
+        LoadState => &mut mapping_config.load_state,
+        NextSaveStateSlot => &mut mapping_config.next_save_state_slot,
+        PrevSaveStateSlot => &mut mapping_config.prev_save_state_slot,
+        SoftReset => &mut mapping_config.soft_reset,
+        HardReset => &mut mapping_config.hard_reset,
+        Pause => &mut mapping_config.pause,
+        StepFrame => &mut mapping_config.step_frame,
+        FastForward => &mut mapping_config.fast_forward,
+        Rewind => &mut mapping_config.rewind,
+        ToggleOverclocking => &mut mapping_config.toggle_overclocking,
+        OpenDebugger => &mut mapping_config.open_debugger,
+        SaveStateSlot0 => &mut mapping_config.save_state_slot_0,
+        SaveStateSlot1 => &mut mapping_config.save_state_slot_1,
+        SaveStateSlot2 => &mut mapping_config.save_state_slot_2,
+        SaveStateSlot3 => &mut mapping_config.save_state_slot_3,
+        SaveStateSlot4 => &mut mapping_config.save_state_slot_4,
+        SaveStateSlot5 => &mut mapping_config.save_state_slot_5,
+        SaveStateSlot6 => &mut mapping_config.save_state_slot_6,
+        SaveStateSlot7 => &mut mapping_config.save_state_slot_7,
+        SaveStateSlot8 => &mut mapping_config.save_state_slot_8,
+        SaveStateSlot9 => &mut mapping_config.save_state_slot_9,
+        LoadStateSlot0 => &mut mapping_config.load_state_slot_0,
+        LoadStateSlot1 => &mut mapping_config.load_state_slot_1,
+        LoadStateSlot2 => &mut mapping_config.load_state_slot_2,
+        LoadStateSlot3 => &mut mapping_config.load_state_slot_3,
+        LoadStateSlot4 => &mut mapping_config.load_state_slot_4,
+        LoadStateSlot5 => &mut mapping_config.load_state_slot_5,
+        LoadStateSlot6 => &mut mapping_config.load_state_slot_6,
+        LoadStateSlot7 => &mut mapping_config.load_state_slot_7,
+        LoadStateSlot8 => &mut mapping_config.load_state_slot_8,
+        LoadStateSlot9 => &mut mapping_config.load_state_slot_9,
     }
 }
 
@@ -1094,7 +1098,7 @@ impl HotkeyExt for Hotkey {
 
         match self {
             PowerOff | Exit | ToggleFullscreen | SoftReset | HardReset | Pause | StepFrame
-            | FastForward | Rewind | OpenDebugger => HotkeyCategory::General,
+            | FastForward | Rewind | ToggleOverclocking | OpenDebugger => HotkeyCategory::General,
             SaveState | LoadState | NextSaveStateSlot | PrevSaveStateSlot | SaveStateSlot0
             | SaveStateSlot1 | SaveStateSlot2 | SaveStateSlot3 | SaveStateSlot4
             | SaveStateSlot5 | SaveStateSlot6 | SaveStateSlot7 | SaveStateSlot8
