@@ -8,10 +8,11 @@ use jgenesis_native_driver::input::{
     AxisDirection, GamepadAction, GenericInput, HatDirection, Joysticks,
 };
 use jgenesis_native_driver::{
-    AudioError, Native32XEmulator, NativeEmulatorResult, NativeGameBoyEmulator,
-    NativeGenesisEmulator, NativeNesEmulator, NativeSegaCdEmulator, NativeSmsGgEmulator,
-    NativeSnesEmulator, NativeTickEffect, SaveStateMetadata,
+    AudioError, Native32XEmulator, NativeEmulatorError, NativeEmulatorResult,
+    NativeGameBoyEmulator, NativeGenesisEmulator, NativeNesEmulator, NativeSegaCdEmulator,
+    NativeSmsGgEmulator, NativeSnesEmulator, NativeTickEffect, SaveStateMetadata,
 };
+use jgenesis_proc_macros::MatchEachVariantMacro;
 use sdl2::EventPump;
 use sdl2::event::Event;
 use sdl2::joystick::{HatState, Joystick};
@@ -20,7 +21,7 @@ use std::collections::HashSet;
 use std::path::PathBuf;
 use std::sync::atomic::{AtomicBool, AtomicU8, Ordering};
 use std::sync::mpsc::{Receiver, Sender, TryRecvError};
-use std::sync::{Arc, Mutex, MutexGuard, mpsc};
+use std::sync::{Arc, Mutex, mpsc};
 use std::thread;
 use std::time::Duration;
 
@@ -83,8 +84,8 @@ impl Console {
 
 #[derive(Debug, Clone)]
 pub enum EmuThreadCommand {
-    Run { console: Console, config: Box<AppConfig>, file_path: String },
-    ReloadConfig(Box<AppConfig>, String),
+    Run { console: Console, config: Box<AppConfig>, file_path: PathBuf },
+    ReloadConfig(Box<AppConfig>, PathBuf),
     StopEmulator,
     CollectInput { axis_deadzone: i16 },
     SoftReset,
@@ -101,7 +102,7 @@ pub struct EmuThreadHandle {
     command_sender: Sender<EmuThreadCommand>,
     input_receiver: Receiver<Option<Vec<GenericInput>>>,
     save_state_metadata: Arc<Mutex<SaveStateMetadata>>,
-    emulator_error: Arc<Mutex<Option<anyhow::Error>>>,
+    emulator_error: Arc<Mutex<Option<NativeEmulatorError>>>,
     exit_signal: Arc<AtomicBool>,
 }
 
@@ -118,8 +119,8 @@ impl EmuThreadHandle {
         self.save_state_metadata.lock().unwrap().clone()
     }
 
-    pub fn lock_emulator_error(&mut self) -> MutexGuard<'_, Option<anyhow::Error>> {
-        self.emulator_error.lock().unwrap()
+    pub fn emulator_error(&self) -> Arc<Mutex<Option<NativeEmulatorError>>> {
+        Arc::clone(&self.emulator_error)
     }
 
     pub fn poll_input_receiver(&self) -> Result<Option<Vec<GenericInput>>, TryRecvError> {
@@ -188,7 +189,7 @@ struct EmuThreadContext {
     input_sender: Sender<Option<Vec<GenericInput>>>,
     status: Arc<AtomicU8>,
     save_state_metadata: Arc<Mutex<SaveStateMetadata>>,
-    emulator_error: Arc<Mutex<Option<anyhow::Error>>>,
+    emulator_error: Arc<Mutex<Option<NativeEmulatorError>>>,
     exit_signal: Arc<AtomicBool>,
 }
 
@@ -216,7 +217,7 @@ fn thread_run(ctx: EmuThreadContext) {
                     Ok(emulator) => emulator,
                     Err(err) => {
                         log::error!("Error initializing emulator: {err}");
-                        *ctx.emulator_error.lock().unwrap() = Some(err.into());
+                        *ctx.emulator_error.lock().unwrap() = Some(err);
                         continue;
                     }
                 };
@@ -254,6 +255,7 @@ fn thread_run(ctx: EmuThreadContext) {
     }
 }
 
+#[derive(MatchEachVariantMacro)]
 enum GenericEmulator {
     SmsGg(NativeSmsGgEmulator),
     Genesis(NativeGenesisEmulator),
@@ -264,25 +266,11 @@ enum GenericEmulator {
     GameBoy(NativeGameBoyEmulator),
 }
 
-macro_rules! match_each_emulator_variant {
-    ($value:expr, $emulator:ident => $expr:expr) => {
-        match $value {
-            GenericEmulator::SmsGg($emulator) => $expr,
-            GenericEmulator::Genesis($emulator) => $expr,
-            GenericEmulator::SegaCd($emulator) => $expr,
-            GenericEmulator::Sega32X($emulator) => $expr,
-            GenericEmulator::Nes($emulator) => $expr,
-            GenericEmulator::Snes($emulator) => $expr,
-            GenericEmulator::GameBoy($emulator) => $expr,
-        }
-    };
-}
-
 impl GenericEmulator {
     fn create(
         console: Console,
         config: Box<AppConfig>,
-        path: String,
+        path: PathBuf,
     ) -> NativeEmulatorResult<Self> {
         let emulator = match console {
             Console::MasterSystem | Console::GameGear => {
@@ -309,7 +297,7 @@ impl GenericEmulator {
         Ok(emulator)
     }
 
-    fn reload_config(&mut self, config: Box<AppConfig>, path: String) -> Result<(), AudioError> {
+    fn reload_config(&mut self, config: Box<AppConfig>, path: PathBuf) -> Result<(), AudioError> {
         match self {
             Self::SmsGg(emulator) => emulator.reload_smsgg_config(config.smsgg_config(path)),
             Self::Genesis(emulator) => emulator.reload_genesis_config(config.genesis_config(path)),
@@ -336,45 +324,43 @@ impl GenericEmulator {
     }
 
     fn render_frame(&mut self) -> NativeEmulatorResult<Option<NativeTickEffect>> {
-        match_each_emulator_variant!(self, emulator => emulator.render_frame())
+        match_each_variant!(self, emulator => emulator.render_frame())
     }
 
     fn soft_reset(&mut self) {
-        match_each_emulator_variant!(self, emulator => emulator.soft_reset());
+        match_each_variant!(self, emulator => emulator.soft_reset());
     }
 
     fn hard_reset(&mut self) {
-        match_each_emulator_variant!(self, emulator => emulator.hard_reset());
+        match_each_variant!(self, emulator => emulator.hard_reset());
     }
 
     fn open_memory_viewer(&mut self) {
-        match_each_emulator_variant!(self, emulator => emulator.open_memory_viewer());
+        match_each_variant!(self, emulator => emulator.open_memory_viewer());
     }
 
     fn save_state(&mut self, slot: usize) {
-        if let Err(err) = match_each_emulator_variant!(self, emulator => emulator.save_state(slot))
-        {
+        if let Err(err) = match_each_variant!(self, emulator => emulator.save_state(slot)) {
             log::error!("Failed to save state to slot {slot}: {err}");
         }
     }
 
     fn load_state(&mut self, slot: usize) {
-        if let Err(err) = match_each_emulator_variant!(self, emulator => emulator.load_state(slot))
-        {
+        if let Err(err) = match_each_variant!(self, emulator => emulator.load_state(slot)) {
             log::error!("Failed to load state from slot {slot}: {err}");
         }
     }
 
     fn save_state_metadata(&self) -> SaveStateMetadata {
-        match_each_emulator_variant!(self, emulator => emulator.save_state_metadata().clone())
+        match_each_variant!(self, emulator => emulator.save_state_metadata().clone())
     }
 
     fn focus(&mut self) {
-        match_each_emulator_variant!(self, emulator => emulator.focus());
+        match_each_variant!(self, emulator => emulator.focus());
     }
 
     fn event_pump_and_joysticks_mut(&mut self) -> (&mut EventPump, &mut Joysticks) {
-        match_each_emulator_variant!(self, emulator => emulator.event_pump_and_joysticks_mut())
+        match_each_variant!(self, emulator => emulator.event_pump_and_joysticks_mut())
     }
 }
 
@@ -439,7 +425,7 @@ fn run_emulator(mut emulator: GenericEmulator, ctx: &EmuThreadContext) {
             }
             Err(err) => {
                 log::error!("Emulator terminated with an error: {err}");
-                *ctx.emulator_error.lock().unwrap() = Some(err.into());
+                *ctx.emulator_error.lock().unwrap() = Some(err);
                 return;
             }
         }
@@ -693,7 +679,7 @@ fn collect_input(
             }
         }
 
-        jgenesis_common::sleep(Duration::from_millis(10));
+        thread::sleep(Duration::from_millis(10));
     }
 }
 
