@@ -104,7 +104,7 @@ const fn default_audio_output_frequency() -> u64 {
 }
 
 const fn default_audio_hardware_queue_size() -> u16 {
-    256
+    512
 }
 
 const fn default_audio_buffer_size() -> u32 {
@@ -142,7 +142,7 @@ fn default_rewind_buffer_length() -> u64 {
 
 impl AppConfig {
     #[must_use]
-    pub fn common_config(&self, path: String) -> CommonConfig {
+    pub fn common_config(&self, path: PathBuf) -> CommonConfig {
         CommonConfig {
             rom_file_path: path,
             audio_output_frequency: self.common.audio_output_frequency,
