@@ -161,7 +161,7 @@ pub struct SmsGgInputMapping {
     pub p1: SmsGgControllerMapping,
     #[serde(default)]
     pub p2: SmsGgControllerMapping,
-    #[debug_fmt]
+    #[cfg_display(debug_fmt)]
     pub pause: Option<Vec<GenericInput>>,
 }
 
@@ -179,10 +179,10 @@ impl SmsGgInputMapping {
 #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize, ConfigDisplay)]
 pub struct SmsGgInputConfig {
     #[serde(default = "default_smsgg_mapping_1")]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_1: SmsGgInputMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_2: SmsGgInputMapping,
 }
 
@@ -260,10 +260,10 @@ impl GenesisControllerMapping {
 #[derive(Debug, Clone, PartialEq, Eq, Default, Serialize, Deserialize, ConfigDisplay)]
 pub struct GenesisInputMapping {
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub p1: GenesisControllerMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub p2: GenesisControllerMapping,
 }
 
@@ -281,10 +281,10 @@ pub struct GenesisInputConfig {
     #[serde(default)]
     pub p2_type: GenesisControllerType,
     #[serde(default = "default_genesis_mapping_1")]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_1: GenesisInputMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_2: GenesisInputMapping,
 }
 
@@ -369,13 +369,13 @@ impl NesZapperMapping {
 #[derive(Debug, Clone, PartialEq, Eq, Default, Serialize, Deserialize, ConfigDisplay)]
 pub struct NesInputMapping {
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub p1: NesControllerMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub p2: NesControllerMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub zapper: NesZapperMapping,
 }
 
@@ -402,10 +402,10 @@ pub struct NesInputConfig {
     #[serde(default)]
     pub p2_type: NesControllerType,
     #[serde(default = "default_nes_mapping_1")]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_1: NesInputMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_2: NesInputMapping,
 }
 
@@ -506,13 +506,13 @@ impl SnesSuperScopeMapping {
 #[derive(Debug, Clone, PartialEq, Eq, Default, Serialize, Deserialize, ConfigDisplay)]
 pub struct SnesInputMapping {
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub p1: SnesControllerMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub p2: SnesControllerMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub super_scope: SnesSuperScopeMapping,
 }
 
@@ -539,10 +539,10 @@ pub struct SnesInputConfig {
     #[serde(default)]
     pub p2_type: SnesControllerType,
     #[serde(default = "default_snes_mapping_1")]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_1: SnesInputMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_2: SnesInputMapping,
 }
 
@@ -612,10 +612,10 @@ impl GameBoyInputMapping {
 #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize, ConfigDisplay)]
 pub struct GameBoyInputConfig {
     #[serde(default = "default_gb_mapping_1")]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_1: GameBoyInputMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_2: GameBoyInputMapping,
 }
 
@@ -707,6 +707,7 @@ define_hotkey_mapping!(
     step_frame: StepFrame default N,
     fast_forward: FastForward default Tab,
     rewind: Rewind default Backquote,
+    toggle_overclocking: ToggleOverclocking default Semicolon,
     open_debugger: OpenDebugger default Quote,
     save_state_slot_0: SaveStateSlot0 default none,
     save_state_slot_1: SaveStateSlot1 default none,
@@ -733,10 +734,10 @@ define_hotkey_mapping!(
 #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize, ConfigDisplay)]
 pub struct HotkeyConfig {
     #[serde(default = "default_hotkey_mapping_1")]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_1: HotkeyMapping,
     #[serde(default)]
-    #[indent_nested]
+    #[cfg_display(indent_nested)]
     pub mapping_2: HotkeyMapping,
 }
 
