@@ -23,7 +23,7 @@ use crate::archive::ArchiveError;
 use crate::config::input::ButtonMappingVec;
 use crate::config::{CommonConfig, FullscreenMode, HideMouseCursor, WindowSize};
 use crate::fpstracker::FpsTracker;
-use crate::input::{CompactHotkey, Hotkey, HotkeyEvent, InputMapper, Joysticks, MappableInputs};
+use crate::input::{CompactHotkey, Hotkey, HotkeyEvent, InputMapper, Joysticks};
 use crate::mainloop::audio::SdlAudioOutput;
 use crate::mainloop::debug::{DebugRenderFn, DebuggerWindow};
 use crate::mainloop::rewind::Rewinder;
@@ -32,7 +32,7 @@ use crate::mainloop::state::SaveStatePaths;
 pub use audio::AudioError;
 use bincode::error::{DecodeError, EncodeError};
 use gb_core::api::GameBoyLoadError;
-use jgenesis_common::frontend::{EmulatorTrait, PartialClone, TickEffect};
+use jgenesis_common::frontend::{EmulatorConfigTrait, EmulatorTrait, TickEffect};
 use jgenesis_renderer::renderer;
 use jgenesis_renderer::renderer::{RendererError, WgpuRenderer};
 use nes_core::api::NesInitializationError;
@@ -47,12 +47,10 @@ use std::cell::RefCell;
 use std::error::Error;
 use std::ffi::NulError;
 use std::fmt::Debug;
-use std::hash::Hash;
-use std::io;
 use std::path::{Path, PathBuf};
 use std::rc::Rc;
-use std::sync::LazyLock;
 use std::time::Duration;
+use std::{io, thread};
 use thiserror::Error;
 
 const MODAL_DURATION: Duration = Duration::from_secs(3);
@@ -133,33 +131,21 @@ struct HotkeyState<Emulator> {
     should_step_frame: bool,
     fast_forward_multiplier: u64,
     rewinder: Rewinder<Emulator>,
+    overclocking_enabled: bool,
     debugger_window: Option<DebuggerWindow<Emulator>>,
     window_scale_factor: Option<f32>,
     debug_render_fn: fn() -> Box<DebugRenderFn<Emulator>>,
 }
 
-impl<Emulator> HotkeyState<Emulator> {
-    fn update_save_state_path(&mut self, save_state_path: PathBuf) -> NativeEmulatorResult<()> {
-        if save_state_path == self.base_save_state_path {
-            return Ok(());
-        }
-
-        self.save_state_paths = state::init_paths(&save_state_path)?;
-        self.save_state_metadata = SaveStateMetadata::load(&self.save_state_paths);
-        self.base_save_state_path = save_state_path;
-
-        Ok(())
-    }
-}
-
-impl<Emulator: PartialClone> HotkeyState<Emulator> {
+impl<Emulator: EmulatorTrait> HotkeyState<Emulator> {
     fn new(
         common_config: &CommonConfig,
         save_state_path: PathBuf,
         debug_render_fn: fn() -> Box<DebugRenderFn<Emulator>>,
     ) -> NativeEmulatorResult<Self> {
         let save_state_paths = state::init_paths(&save_state_path)?;
-        let save_state_metadata = SaveStateMetadata::load(&save_state_paths);
+        let save_state_metadata =
+            SaveStateMetadata::load(&save_state_paths, Emulator::save_state_version());
 
         log::debug!("Save state paths: {save_state_paths:?}");
 
@@ -176,11 +162,25 @@ impl<Emulator: PartialClone> HotkeyState<Emulator> {
             rewinder: Rewinder::new(Duration::from_secs(
                 common_config.rewind_buffer_length_seconds,
             )),
+            overclocking_enabled: true,
             debugger_window: None,
             window_scale_factor: common_config.window_scale_factor,
             debug_render_fn,
         })
     }
+
+    fn update_save_state_path(&mut self, save_state_path: PathBuf) -> NativeEmulatorResult<()> {
+        if save_state_path == self.base_save_state_path {
+            return Ok(());
+        }
+
+        self.save_state_paths = state::init_paths(&save_state_path)?;
+        self.save_state_metadata =
+            SaveStateMetadata::load(&self.save_state_paths, Emulator::save_state_version());
+        self.base_save_state_path = save_state_path;
+
+        Ok(())
+    }
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -189,12 +189,15 @@ pub enum NativeTickEffect {
     Exit,
 }
 
-pub struct NativeEmulator<Inputs, Button, Config, Emulator> {
+pub struct NativeEmulator<Emulator: EmulatorTrait> {
     emulator: Emulator,
-    config: Config,
+    // Config sent from the frontend
+    raw_config: Emulator::Config,
+    // Config with overclocking maybe forcibly disabled due to hotkey state
+    config: Emulator::Config,
     renderer: WgpuRenderer<Window>,
     audio_output: SdlAudioOutput,
-    input_mapper: InputMapper<Inputs, Button>,
+    input_mapper: InputMapper<Emulator::Inputs, Emulator::Button>,
     save_writer: FsSaveWriter,
     sdl: Sdl,
     event_pump: EventPump,
@@ -206,9 +209,7 @@ pub struct NativeEmulator<Inputs, Button, Config, Emulator> {
     rom_extension: String,
 }
 
-impl<Inputs, Button, Config, Emulator: EmulatorTrait>
-    NativeEmulator<Inputs, Button, Config, Emulator>
-{
+impl<Emulator: EmulatorTrait> NativeEmulator<Emulator> {
     fn reload_common_config(&mut self, config: &CommonConfig) -> Result<(), AudioError> {
         self.renderer.reload_config(config.renderer_config);
 
@@ -350,7 +351,7 @@ pub enum NativeEmulatorError {
     SegaCdNoBios,
     #[error("Error opening BIOS file at '{path}': {source}")]
     SegaCdBiosRead {
-        path: String,
+        path: PathBuf,
         #[source]
         source: io::Error,
     },
@@ -371,20 +372,25 @@ pub enum NativeEmulatorError {
     },
     #[error("Error saving state: {0}")]
     SaveState(#[from] EncodeError),
+    #[error("Error saving state: {0}")]
+    SaveStateIo(io::Error),
     #[error("Error loading state: {0}")]
     LoadState(#[from] DecodeError),
+    #[error("Error loading state: {0}")]
+    LoadStateIo(io::Error),
+    #[error("Save state begins with invalid prefix")]
+    LoadStatePrefixMismatch,
+    #[error("Save state version mismatch; expected {expected}, got {actual}")]
+    LoadStateVersionMismatch { expected: u16, actual: u16 },
     #[error("Error in emulation core: {0}")]
     Emulator(#[source] Box<dyn Error + Send + Sync + 'static>),
 }
 
 pub type NativeEmulatorResult<T> = Result<T, NativeEmulatorError>;
 
-// TODO simplify or generalize these trait bounds
-impl<Inputs, Button, Config, Emulator> NativeEmulator<Inputs, Button, Config, Emulator>
+impl<Emulator> NativeEmulator<Emulator>
 where
-    Inputs: Default + MappableInputs<Button>,
-    Button: Debug + Copy + Hash + Eq,
-    Emulator: EmulatorTrait<Inputs = Inputs, Config = Config>,
+    Emulator: EmulatorTrait,
 {
     #[allow(clippy::too_many_arguments)]
     fn new(
@@ -396,8 +402,8 @@ where
         window_title: &str,
         save_writer: FsSaveWriter,
         save_state_path: PathBuf,
-        button_mappings: &ButtonMappingVec<'_, Button>,
-        initial_inputs: Inputs,
+        button_mappings: &ButtonMappingVec<'_, Emulator::Button>,
+        initial_inputs: Emulator::Inputs,
         debug_render_fn: fn() -> Box<DebugRenderFn<Emulator>>,
     ) -> NativeEmulatorResult<Self> {
         let (sdl, video, audio, joystick, event_pump) = init_sdl(&common_config)?;
@@ -438,6 +444,7 @@ where
 
         let mut emulator = Self {
             emulator,
+            raw_config: emulator_config.clone(),
             config: emulator_config,
             renderer,
             audio_output,
@@ -449,7 +456,7 @@ where
             video,
             hotkey_state,
             fps_tracker: FpsTracker::new(),
-            rom_path: common_config.rom_file_path.into(),
+            rom_path: common_config.rom_file_path,
             rom_extension,
         };
 
@@ -570,7 +577,7 @@ where
 
         if !should_run_emulator {
             // Don't spin loop when the emulator is paused or rewinding
-            jgenesis_common::sleep(Duration::from_millis(1));
+            thread::sleep(Duration::from_millis(1));
         }
 
         Ok(None)
@@ -694,108 +701,101 @@ where
         match hotkey {
             CompactHotkey::PowerOff => return Ok(Some(HotkeyEffect::PowerOff)),
             CompactHotkey::Exit => return Ok(Some(HotkeyEffect::Exit)),
-            CompactHotkey::ToggleFullscreen => {
-                self.renderer
-                    .toggle_fullscreen(self.hotkey_state.fullscreen_mode)
-                    .map_err(NativeEmulatorError::SdlSetFullscreen)?;
-                self.sdl.mouse().show_cursor(
-                    !self.hotkey_state.hide_mouse_cursor.should_hide(self.renderer.is_fullscreen()),
-                );
-            }
-            CompactHotkey::SaveState | CompactHotkey::SaveStateSlot(..) => {
-                let slot = match hotkey {
-                    CompactHotkey::SaveState => self.hotkey_state.save_state_slot,
-                    CompactHotkey::SaveStateSlot(slot) => slot,
-                    _ => unreachable!("nested match expressions"),
-                };
-                self.save_state(slot)?;
-                log::info!(
-                    "Saved state to slot {slot} in '{}'",
-                    self.hotkey_state.save_state_paths[slot].display()
-                );
-            }
-            CompactHotkey::LoadState | CompactHotkey::LoadStateSlot(..) => {
-                let slot = match hotkey {
-                    CompactHotkey::LoadState => self.hotkey_state.save_state_slot,
-                    CompactHotkey::LoadStateSlot(slot) => slot,
-                    _ => unreachable!("nested match expressions"),
-                };
-
-                match self.load_state(slot) {
-                    Ok(()) => {
-                        log::info!(
-                            "Loaded state from slot {slot} in '{}'",
-                            self.hotkey_state.save_state_paths[slot].display()
-                        );
-                    }
-                    Err(err) => {
-                        log::error!(
-                            "Error loading save state from slot {slot} in '{}': {err}",
-                            self.hotkey_state.save_state_paths[slot].display()
-                        );
-                    }
-                }
-            }
-            CompactHotkey::SoftReset => {
-                self.emulator.soft_reset();
-            }
-            CompactHotkey::HardReset => {
-                self.emulator.hard_reset(&mut self.save_writer);
-            }
-            CompactHotkey::NextSaveStateSlot => {
-                self.hotkey_state.save_state_slot =
-                    (self.hotkey_state.save_state_slot + 1) % SAVE_STATE_SLOTS;
-                self.renderer.add_modal(
-                    format!("Selected save state slot {}", self.hotkey_state.save_state_slot),
-                    MODAL_DURATION,
-                );
-            }
-            CompactHotkey::PrevSaveStateSlot => {
-                self.hotkey_state.save_state_slot = if self.hotkey_state.save_state_slot == 0 {
-                    state::SAVE_STATE_SLOTS - 1
-                } else {
-                    self.hotkey_state.save_state_slot - 1
-                };
-                self.renderer.add_modal(
-                    format!("Selected save state slot {}", self.hotkey_state.save_state_slot),
-                    MODAL_DURATION,
-                );
-            }
+            CompactHotkey::ToggleFullscreen => self.toggle_fullscreen()?,
+            CompactHotkey::SaveState => self.save_state(self.hotkey_state.save_state_slot)?,
+            CompactHotkey::SaveStateSlot(slot) => self.save_state(slot)?,
+            CompactHotkey::LoadState => self.hotkey_load_state(None),
+            CompactHotkey::LoadStateSlot(slot) => self.hotkey_load_state(Some(slot)),
+            CompactHotkey::SoftReset => self.emulator.soft_reset(),
+            CompactHotkey::HardReset => self.emulator.hard_reset(&mut self.save_writer),
+            CompactHotkey::NextSaveStateSlot => self.next_save_state_slot(),
+            CompactHotkey::PrevSaveStateSlot => self.prev_save_state_slot(),
             CompactHotkey::Pause => {
                 self.hotkey_state.paused = !self.hotkey_state.paused;
             }
             CompactHotkey::StepFrame => {
                 self.hotkey_state.should_step_frame = true;
             }
-            CompactHotkey::FastForward => {
-                let multiplier = self.hotkey_state.fast_forward_multiplier;
-                self.renderer.set_speed_multiplier(multiplier);
-                self.audio_output.set_speed_multiplier(multiplier);
-            }
-            CompactHotkey::Rewind => {
-                self.hotkey_state.rewinder.start_rewinding();
-            }
-            CompactHotkey::OpenDebugger => {
-                if self.hotkey_state.debugger_window.is_none() {
-                    let debug_render_fn = (self.hotkey_state.debug_render_fn)();
-                    match DebuggerWindow::new(
-                        &self.video,
-                        self.hotkey_state.window_scale_factor,
-                        debug_render_fn,
-                    ) {
-                        Ok(debugger_window) => {
-                            self.hotkey_state.debugger_window = Some(debugger_window);
-                        }
-                        Err(err) => {
-                            log::error!("Error opening debugger window: {err}");
-                        }
-                    }
-                }
-            }
+            CompactHotkey::FastForward => self.enable_fast_forward(),
+            CompactHotkey::Rewind => self.hotkey_state.rewinder.start_rewinding(),
+            CompactHotkey::ToggleOverclocking => self.toggle_overclocking(),
+            CompactHotkey::OpenDebugger => self.open_memory_viewer(),
         }
 
         Ok(None)
     }
+
+    fn toggle_fullscreen(&mut self) -> NativeEmulatorResult<()> {
+        self.renderer
+            .toggle_fullscreen(self.hotkey_state.fullscreen_mode)
+            .map_err(NativeEmulatorError::SdlSetFullscreen)?;
+        self.sdl.mouse().show_cursor(
+            !self.hotkey_state.hide_mouse_cursor.should_hide(self.renderer.is_fullscreen()),
+        );
+
+        Ok(())
+    }
+
+    fn hotkey_load_state(&mut self, slot: Option<usize>) {
+        let slot = slot.unwrap_or(self.hotkey_state.save_state_slot);
+
+        if let Err(err) = self.load_state(slot) {
+            log::error!(
+                "Error loading save state from slot {slot} in '{}': {err}",
+                self.hotkey_state.save_state_paths[slot].display()
+            );
+        }
+    }
+
+    fn next_save_state_slot(&mut self) {
+        self.hotkey_state.save_state_slot =
+            (self.hotkey_state.save_state_slot + 1) % SAVE_STATE_SLOTS;
+        self.renderer.add_modal(
+            format!("Selected save state slot {}", self.hotkey_state.save_state_slot),
+            MODAL_DURATION,
+        );
+    }
+
+    fn prev_save_state_slot(&mut self) {
+        self.hotkey_state.save_state_slot = if self.hotkey_state.save_state_slot == 0 {
+            SAVE_STATE_SLOTS - 1
+        } else {
+            self.hotkey_state.save_state_slot - 1
+        };
+        self.renderer.add_modal(
+            format!("Selected save state slot {}", self.hotkey_state.save_state_slot),
+            MODAL_DURATION,
+        );
+    }
+
+    fn enable_fast_forward(&mut self) {
+        let multiplier = self.hotkey_state.fast_forward_multiplier;
+        self.renderer.set_speed_multiplier(multiplier);
+        self.audio_output.set_speed_multiplier(multiplier);
+    }
+
+    fn toggle_overclocking(&mut self) {
+        self.hotkey_state.overclocking_enabled = !self.hotkey_state.overclocking_enabled;
+        self.update_emulator_config(&self.raw_config.clone());
+
+        let modal_text = if self.hotkey_state.overclocking_enabled {
+            "Overclocking settings enabled"
+        } else {
+            "Overclocking settings disabled"
+        };
+        self.renderer.add_modal(modal_text.into(), MODAL_DURATION);
+    }
+
+    fn update_emulator_config(&mut self, config: &Emulator::Config) {
+        self.raw_config = config.clone();
+        self.config = if self.hotkey_state.overclocking_enabled {
+            self.raw_config.clone()
+        } else {
+            self.raw_config.with_overclocking_disabled()
+        };
+
+        self.emulator.reload_config(&self.config);
+    }
 }
 
 fn file_name_no_ext<P: AsRef<Path>>(path: P) -> NativeEmulatorResult<String> {
@@ -864,21 +864,3 @@ macro_rules! bincode_config {
 }
 
 use bincode_config;
-
-#[must_use]
-pub fn all_supported_extensions() -> &'static [&'static str] {
-    static EXTENSIONS: LazyLock<Vec<&'static str>> = LazyLock::new(|| {
-        let mut extensions = Vec::new();
-
-        extensions.extend(gb::SUPPORTED_EXTENSIONS);
-        extensions.extend(genesis::GENESIS_SUPPORTED_EXTENSIONS);
-        extensions.extend(genesis::S32X_SUPPORTED_EXTENSIONS);
-        extensions.extend(nes::SUPPORTED_EXTENSIONS);
-        extensions.extend(smsgg::SUPPORTED_EXTENSIONS);
-        extensions.extend(snes::SUPPORTED_EXTENSIONS);
-
-        extensions
-    });
-
-    &EXTENSIONS
-}
