@@ -3,6 +3,7 @@ use jgenesis_common::audio::DynamicResamplingRate;
 use jgenesis_common::frontend::AudioOutput;
 use sdl2::AudioSubsystem;
 use sdl2::audio::{AudioQueue, AudioSpecDesired};
+use std::thread;
 use std::time::Duration;
 use thiserror::Error;
 
@@ -118,11 +119,14 @@ fn open_audio_queue(
     config: &CommonConfig,
 ) -> Result<AudioQueue<f32>, AudioError> {
     let audio_queue = audio
-        .open_queue(None, &AudioSpecDesired {
-            freq: Some(config.audio_output_frequency as i32),
-            channels: Some(CHANNELS),
-            samples: Some(config.audio_hardware_queue_size),
-        })
+        .open_queue(
+            None,
+            &AudioSpecDesired {
+                freq: Some(config.audio_output_frequency as i32),
+                channels: Some(CHANNELS),
+                samples: Some(config.audio_hardware_queue_size),
+            },
+        )
         .map_err(AudioError::OpenQueue)?;
     audio_queue.resume();
 
@@ -167,7 +171,7 @@ impl AudioOutput for SdlAudioOutput {
             if self.audio_sync {
                 // Block until audio queue is not full
                 while self.audio_queue_len_samples() > audio_buffer_threshold {
-                    jgenesis_common::sleep(Duration::from_micros(250));
+                    thread::sleep(Duration::from_micros(250));
                 }
             } else if self.audio_queue_len_samples() > audio_buffer_threshold {
                 // Audio queue is full; drop samples
