@@ -1,12 +1,11 @@
 use crate::config::{PreprocessShader, PrescaleMode, RendererConfig, Scanlines, WgpuBackend};
 use cfg_if::cfg_if;
-use jgenesis_common::frontend::{Color, FrameSize, PixelAspectRatio, Renderer};
+use jgenesis_common::frontend::{Color, DisplayArea, FrameSize, PixelAspectRatio, Renderer};
 use jgenesis_common::timeutils;
 use raw_window_handle::{HandleError, HasDisplayHandle, HasWindowHandle};
 use std::collections::HashMap;
 use std::error::Error;
 use std::fmt::Debug;
-use std::time::Duration;
 use std::{cmp, iter};
 use thiserror::Error;
 use wgpu::util::DeviceExt;
@@ -223,7 +222,7 @@ fn create_horizontal_blur_pipeline(
         layout: Some(&pipeline_layout),
         vertex: wgpu::VertexState {
             module: &shaders.identity,
-            entry_point: "vs_main",
+            entry_point: None,
             compilation_options: wgpu::PipelineCompilationOptions::default(),
             buffers: &[],
         },
@@ -244,7 +243,7 @@ fn create_horizontal_blur_pipeline(
         },
         fragment: Some(wgpu::FragmentState {
             module: &shaders.hblur,
-            entry_point: fs_main,
+            entry_point: Some(fs_main),
             compilation_options: wgpu::PipelineCompilationOptions::default(),
             targets: &[Some(wgpu::ColorTargetState {
                 format: output_texture.format(),
@@ -461,7 +460,7 @@ impl RenderingPipeline {
             layout: Some(&prescale_pipeline_layout),
             vertex: wgpu::VertexState {
                 module: &shaders.identity,
-                entry_point: "vs_main",
+                entry_point: None,
                 compilation_options: wgpu::PipelineCompilationOptions::default(),
                 buffers: &[],
             },
@@ -482,7 +481,7 @@ impl RenderingPipeline {
             },
             fragment: Some(wgpu::FragmentState {
                 module: &shaders.prescale,
-                entry_point: prescale_fs_main,
+                entry_point: Some(prescale_fs_main),
                 compilation_options: wgpu::PipelineCompilationOptions::default(),
                 targets: &[Some(wgpu::ColorTargetState {
                     format: scaled_texture.format(),
@@ -544,7 +543,7 @@ impl RenderingPipeline {
             layout: Some(&render_pipeline_layout),
             vertex: wgpu::VertexState {
                 module: &shaders.render,
-                entry_point: "vs_main",
+                entry_point: None,
                 compilation_options: wgpu::PipelineCompilationOptions::default(),
                 buffers: &[Vertex::buffer_layout()],
             },
@@ -565,7 +564,7 @@ impl RenderingPipeline {
             },
             fragment: Some(wgpu::FragmentState {
                 module: &shaders.render,
-                entry_point: "fs_main",
+                entry_point: None,
                 compilation_options: wgpu::PipelineCompilationOptions::default(),
                 targets: &[Some(wgpu::ColorTargetState {
                     format: surface_config.format,
@@ -757,14 +756,6 @@ fn compute_vertices(
         .collect()
 }
 
-#[derive(Debug, Clone, Copy)]
-pub struct DisplayArea {
-    pub width: u32,
-    pub height: u32,
-    pub x: u32,
-    pub y: u32,
-}
-
 fn determine_display_area(
     window_width: u32,
     window_height: u32,
@@ -957,15 +948,15 @@ impl FrameTimeTracker {
             return;
         }
 
-        let mut now = timeutils::current_time_nanos();
         let next_frame_time = self.last_frame_time_nanos + self.frame_interval_nanos;
-        while now < next_frame_time {
-            jgenesis_common::sleep(Duration::from_micros(250));
-            now = timeutils::current_time_nanos();
-        }
+        let now = timeutils::sleep_until(next_frame_time);
+        self.last_frame_time_nanos += self.frame_interval_nanos;
 
-        while self.last_frame_time_nanos + self.frame_interval_nanos < now {
-            self.last_frame_time_nanos += self.frame_interval_nanos;
+        if now > self.last_frame_time_nanos
+            && (now - self.last_frame_time_nanos) > 5 * self.frame_interval_nanos
+        {
+            log::warn!("Frame time sync is more than 5 frames behind; catching up frame time");
+            self.last_frame_time_nanos = now;
         }
     }
 }
@@ -1211,7 +1202,7 @@ impl<Window> WgpuRenderer<Window> {
     }
 
     #[cfg(feature = "ttf")]
-    pub fn add_modal(&mut self, text: String, duration: Duration) {
+    pub fn add_modal(&mut self, text: String, duration: std::time::Duration) {
         self.modal_renderer.add_modal(text, duration);
     }
 }
