@@ -1,57 +1,20 @@
-use genesis_core::audio::LowPassFilter;
 use genesis_core::input::GenesisControllerType;
-use genesis_core::{GenesisAspectRatio, GenesisEmulatorConfig};
-use jgenesis_common::frontend::{PixelAspectRatio, TimingMode};
-use jgenesis_proc_macros::{EnumDisplay, EnumFromStr};
+use genesis_core::{GenesisAspectRatio, GenesisEmulatorConfig, GenesisLowPassFilter};
+use jgenesis_common::frontend::TimingMode;
 use jgenesis_renderer::config::{
     FilterMode, PreprocessShader, PrescaleFactor, PrescaleMode, RendererConfig, Scanlines,
     VSyncMode, WgpuBackend,
 };
-use smsgg_core::{SmsGgEmulatorConfig, SmsGgHardware, SmsModel, SmsRegion};
+use segacd_core::api::{PcmInterpolation, PcmLowPassFilter, SegaCdEmulatorConfig};
+use smsgg_core::{GgAspectRatio, SmsAspectRatio, SmsGgEmulatorConfig, SmsModel, SmsRegion};
 use snes_core::api::{AudioInterpolationMode, SnesAspectRatio, SnesEmulatorConfig};
 use std::cell::RefCell;
 use std::collections::VecDeque;
-use std::num::{NonZeroU32, NonZeroU64};
+use std::num::{NonZeroU16, NonZeroU32, NonZeroU64};
 use std::ops::Deref;
 use std::rc::Rc;
 use wasm_bindgen::prelude::*;
 
-#[derive(Debug, Clone, Copy, Default, PartialEq, Eq, EnumDisplay, EnumFromStr)]
-enum SmsAspectRatio {
-    #[default]
-    Ntsc,
-    Pal,
-    SquarePixels,
-}
-
-impl SmsAspectRatio {
-    fn to_pixel_aspect_ratio(self) -> PixelAspectRatio {
-        match self {
-            Self::Ntsc => PixelAspectRatio::try_from(smsgg_core::SMS_NTSC_ASPECT_RATIO).unwrap(),
-            Self::Pal => PixelAspectRatio::try_from(smsgg_core::SMS_PAL_ASPECT_RATIO).unwrap(),
-            Self::SquarePixels => PixelAspectRatio::SQUARE,
-        }
-    }
-}
-
-#[derive(Debug, Clone, Copy, Default, PartialEq, Eq, EnumDisplay, EnumFromStr)]
-enum GameGearAspectRatio {
-    #[default]
-    GameGearLcd,
-    SquarePixels,
-}
-
-impl GameGearAspectRatio {
-    fn to_pixel_aspect_ratio(self) -> PixelAspectRatio {
-        match self {
-            Self::GameGearLcd => {
-                PixelAspectRatio::try_from(smsgg_core::GAME_GEAR_LCD_ASPECT_RATIO).unwrap()
-            }
-            Self::SquarePixels => PixelAspectRatio::SQUARE,
-        }
-    }
-}
-
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct CommonWebConfig {
     pub filter_mode: FilterMode,
@@ -90,7 +53,7 @@ impl CommonWebConfig {
 pub struct SmsGgWebConfig {
     timing_mode: TimingMode,
     sms_aspect_ratio: SmsAspectRatio,
-    gg_aspect_ratio: GameGearAspectRatio,
+    gg_aspect_ratio: GgAspectRatio,
     region: SmsRegion,
     remove_sprite_limit: bool,
     sms_crop_vertical_border: bool,
@@ -103,10 +66,10 @@ impl Default for SmsGgWebConfig {
         Self {
             timing_mode: TimingMode::default(),
             sms_aspect_ratio: SmsAspectRatio::default(),
-            gg_aspect_ratio: GameGearAspectRatio::default(),
+            gg_aspect_ratio: GgAspectRatio::default(),
             region: SmsRegion::default(),
             remove_sprite_limit: false,
-            sms_crop_vertical_border: false,
+            sms_crop_vertical_border: true,
             sms_crop_left_border: false,
             fm_unit_enabled: true,
         }
@@ -114,18 +77,13 @@ impl Default for SmsGgWebConfig {
 }
 
 impl SmsGgWebConfig {
-    pub(crate) fn to_emulator_config(&self, hardware: SmsGgHardware) -> SmsGgEmulatorConfig {
-        let pixel_aspect_ratio = match hardware {
-            SmsGgHardware::MasterSystem => self.sms_aspect_ratio.to_pixel_aspect_ratio(),
-            SmsGgHardware::GameGear => self.gg_aspect_ratio.to_pixel_aspect_ratio(),
-        };
-
+    pub(crate) fn to_emulator_config(&self) -> SmsGgEmulatorConfig {
         SmsGgEmulatorConfig {
-            hardware,
             sms_timing_mode: self.timing_mode,
             sms_model: SmsModel::default(),
             forced_psg_version: None,
-            pixel_aspect_ratio: Some(pixel_aspect_ratio),
+            sms_aspect_ratio: self.sms_aspect_ratio,
+            gg_aspect_ratio: self.gg_aspect_ratio,
             sms_region: self.region,
             remove_sprite_limit: self.remove_sprite_limit,
             sms_crop_left_border: self.sms_crop_left_border,
@@ -142,6 +100,7 @@ pub struct GenesisWebConfig {
     aspect_ratio: GenesisAspectRatio,
     remove_sprite_limits: bool,
     emulate_non_linear_vdp_dac: bool,
+    low_pass: GenesisLowPassFilter,
     render_vertical_border: bool,
     render_horizontal_border: bool,
     m68k_divider: u64,
@@ -153,6 +112,7 @@ impl Default for GenesisWebConfig {
             aspect_ratio: GenesisAspectRatio::default(),
             remove_sprite_limits: false,
             emulate_non_linear_vdp_dac: false,
+            low_pass: GenesisLowPassFilter::default(),
             render_vertical_border: false,
             render_horizontal_border: false,
             m68k_divider: genesis_core::timing::NATIVE_M68K_DIVIDER,
@@ -172,6 +132,7 @@ impl GenesisWebConfig {
             remove_sprite_limits: self.remove_sprite_limits,
             m68k_clock_divider: self.m68k_divider,
             emulate_non_linear_vdp_dac: self.emulate_non_linear_vdp_dac,
+            deinterlace: true,
             render_vertical_border: self.render_vertical_border,
             render_horizontal_border: self.render_horizontal_border,
             plane_a_enabled: true,
@@ -181,7 +142,7 @@ impl GenesisWebConfig {
             backdrop_enabled: true,
             quantize_ym2612_output: true,
             emulate_ym2612_ladder_effect: true,
-            low_pass_filter: LowPassFilter::Moderate,
+            low_pass: self.low_pass,
             ym2612_enabled: true,
             psg_enabled: true,
         }
@@ -199,6 +160,7 @@ impl SnesWebConfig {
         SnesEmulatorConfig {
             forced_timing_mode: None,
             aspect_ratio: self.aspect_ratio,
+            deinterlace: true,
             audio_interpolation: self.audio_interpolation,
             audio_60hz_hack: true,
             gsu_overclock_factor: NonZeroU64::new(1).unwrap(),
@@ -214,6 +176,24 @@ pub struct WebConfig {
     pub snes: SnesWebConfig,
 }
 
+impl WebConfig {
+    pub fn to_sega_cd_config(&self) -> SegaCdEmulatorConfig {
+        SegaCdEmulatorConfig {
+            genesis: self.genesis.to_emulator_config(),
+            pcm_interpolation: PcmInterpolation::CubicHermite,
+            enable_ram_cartridge: true,
+            load_disc_into_ram: true,
+            disc_drive_speed: NonZeroU16::new(1).unwrap(),
+            sub_cpu_divider: NonZeroU64::new(segacd_core::api::DEFAULT_SUB_CPU_DIVIDER).unwrap(),
+            pcm_low_pass: PcmLowPassFilter::default(),
+            apply_genesis_lpf_to_pcm: false,
+            apply_genesis_lpf_to_cd_da: false,
+            pcm_enabled: true,
+            cd_audio_enabled: true,
+        }
+    }
+}
+
 #[wasm_bindgen]
 pub struct WebConfigRef(Rc<RefCell<WebConfig>>);
 
@@ -293,6 +273,14 @@ impl WebConfigRef {
         self.borrow_mut().genesis.emulate_non_linear_vdp_dac = emulate_non_linear_dac;
     }
 
+    pub fn set_genesis_emulate_low_pass(&self, emulate_low_pass: bool) {
+        self.borrow_mut().genesis.low_pass = if emulate_low_pass {
+            GenesisLowPassFilter::Model1Va2
+        } else {
+            GenesisLowPassFilter::None
+        };
+    }
+
     pub fn set_genesis_render_vertical_border(&self, render_vertical_border: bool) {
         self.borrow_mut().genesis.render_vertical_border = render_vertical_border;
     }
