@@ -17,7 +17,7 @@ use jgenesis_common::frontend::{
 };
 use jgenesis_renderer::renderer::{WgpuRenderer, WindowSize};
 use rfd::AsyncFileDialog;
-use segacd_core::api::{PcmInterpolation, SegaCdEmulator, SegaCdEmulatorConfig};
+use segacd_core::api::SegaCdEmulator;
 use smsgg_core::{SmsGgEmulator, SmsGgHardware, SmsGgInputs};
 use snes_core::api::{CoprocessorRoms, SnesEmulator};
 use snes_core::input::SnesInputs;
@@ -64,8 +64,9 @@ impl AudioOutput for WebAudioOutput {
             let _ = self.audio_ctx.resume();
         }
 
-        self.audio_queue.push_if_space(sample_l as f32).map_err(|err| format!("{err:?}"))?;
-        self.audio_queue.push_if_space(sample_r as f32).map_err(|err| format!("{err:?}"))?;
+        self.audio_queue
+            .push_if_space((sample_l as f32, sample_r as f32))
+            .map_err(|err| format!("{err:?}"))?;
         Ok(())
     }
 }
@@ -242,7 +243,7 @@ impl Renderer for QueuedFrame {
 #[allow(clippy::large_enum_variant)]
 enum Emulator {
     None(RandomNoiseGenerator),
-    SmsGg(SmsGgEmulator, SmsGgInputs, SmsGgHardware),
+    SmsGg(SmsGgEmulator, SmsGgInputs),
     Genesis(GenesisEmulator, GenesisInputs),
     SegaCd(SegaCdEmulator, GenesisInputs),
     Snes(SnesEmulator, SnesInputs),
@@ -264,7 +265,7 @@ impl Emulator {
                 noise_generator.randomize();
                 noise_generator.render(renderer).expect("Failed to render random noise");
             }
-            Self::SmsGg(emulator, inputs, _) => {
+            Self::SmsGg(emulator, inputs) => {
                 while emulator
                     .tick(renderer, audio_output, inputs, save_writer)
                     .expect("Emulator error")
@@ -326,7 +327,7 @@ impl Emulator {
     fn handle_window_event(&mut self, event: &WindowEvent) {
         match self {
             Self::None(..) => {}
-            Self::SmsGg(_, inputs, _) => {
+            Self::SmsGg(_, inputs) => {
                 handle_smsgg_input(inputs, event);
             }
             Self::Genesis(_, inputs) | Self::SegaCd(_, inputs) => {
@@ -341,21 +342,14 @@ impl Emulator {
     fn reload_config(&mut self, config: &WebConfig) {
         match self {
             Self::None(..) => {}
-            Self::SmsGg(emulator, _, hardware) => {
-                emulator.reload_config(&config.smsgg.to_emulator_config(*hardware));
+            Self::SmsGg(emulator, ..) => {
+                emulator.reload_config(&config.smsgg.to_emulator_config());
             }
             Self::Genesis(emulator, ..) => {
                 emulator.reload_config(&config.genesis.to_emulator_config());
             }
             Self::SegaCd(emulator, ..) => {
-                emulator.reload_config(&SegaCdEmulatorConfig {
-                    genesis: config.genesis.to_emulator_config(),
-                    pcm_interpolation: PcmInterpolation::default(),
-                    enable_ram_cartridge: true,
-                    load_disc_into_ram: true,
-                    pcm_enabled: true,
-                    cd_audio_enabled: true,
-                });
+                emulator.reload_config(&config.to_sega_cd_config());
             }
             Self::Snes(emulator, ..) => {
                 emulator.reload_config(&config.snes.to_emulator_config());
@@ -767,7 +761,8 @@ fn run_event_loop(event_loop: EventLoop<JgenesisUserEvent>, mut state: AppState)
 
 async fn open_file(event_loop_proxy: EventLoopProxy<JgenesisUserEvent>) {
     let file = AsyncFileDialog::new()
-        .add_filter("sms/gg/md", &["sms", "gg", "md", "bin", "sfc", "smc"])
+        .add_filter("Supported Files", &["sms", "gg", "gen", "md", "bin", "smd", "sfc", "smc"])
+        .add_filter("All Types", &["*"])
         .pick_file()
         .await;
     let Some(file) = file else { return };
@@ -849,12 +844,13 @@ fn open_emulator(
             };
             let emulator = SmsGgEmulator::create(
                 rom,
-                config_ref.borrow().smsgg.to_emulator_config(hardware),
+                hardware,
+                config_ref.borrow().smsgg.to_emulator_config(),
                 save_writer,
             );
-            Ok(Emulator::SmsGg(emulator, SmsGgInputs::default(), hardware))
+            Ok(Emulator::SmsGg(emulator, SmsGgInputs::default()))
         }
-        "md" | "bin" => {
+        "gen" | "md" | "bin" | "smd" => {
             js::showGenesisConfig();
 
             let emulator = GenesisEmulator::create(
@@ -870,14 +866,7 @@ fn open_emulator(
             let emulator = SegaCdEmulator::create_in_memory(
                 bios,
                 rom,
-                SegaCdEmulatorConfig {
-                    genesis: config_ref.borrow().genesis.to_emulator_config(),
-                    pcm_interpolation: PcmInterpolation::default(),
-                    enable_ram_cartridge: true,
-                    load_disc_into_ram: true,
-                    pcm_enabled: true,
-                    cd_audio_enabled: true,
-                },
+                config_ref.borrow().to_sega_cd_config(),
                 save_writer,
             )?;
 
